#!/bin/sh

# Testing Script for peg.
# usage:  peg_test00[1-4].t 
# To test fail condition, usage: peg_test00[1-4].t force_fail
# SLEEP_DURATION operation time is 10.  The variable is set to a small number (0.1) for testing.

TESTPATH="$( cd "$( dirname "$0" )" && pwd )" 
source "$TESTPATH/peg_common.sh"

TESTID="peg_test003" 
XPANAME="pegtest"

FORCE_FAIL=0

if test "$1" = "force_fail"
then
  FORCE_FAIL=1
fi

DIFF_TRUE_RESULT=0

export TESTID

# In function set_test_env, set OUTDIR, OUTFILE, OUTFILE2, SAVDIR
#                               LOGNAMEFILE, INDIR, DIFF_FILE

set_test_env

echo 
echo " #*** Test Begin: $TESTID.t"
echo -e "\n"
echo "$TESTID regression"
echo -e "\n"
echo "running" $TESTID

#
# Run tests
#
echo " $TESTID LOG " > ${OUTFILE}
echo "" >> ${OUTFILE}

startTime=`perl -e 'use Time::HiRes qw(time); print time'`

echo "Running test $TESTID"
echo " $TESTID LOG " > $OUTFILE
echo "" >> $OUTFILE

echo "check that taskmonitor and dmlist are available" >> $OUTFILE
if [ ! -x $ASCDS_INSTALL/bin/taskmonitor ]; then
  echo "$TESTID FAILED - taskmonitor not found but needed for testing"
  echo "$TESTID FAILED - taskmonitor not found but needed for testing" >> $OUTFILE
  exit 1
fi 
if [ ! -x $ASCDS_INSTALL/bin/dmlist ]; then
  echo "$TESTID FAILED - dmlist not found but needed for testing"
  echo "$TESTID FAILED - dmlist not found but needed for testing" >> $OUTFILE
  exit 1
fi 

peg --xpaname $XPANAME dmlist &
X=0 
sleep $SLEEP_DURATION 

# wait upto five minutes before exiting

while [ $X -le 300 ]
do
   a=`xpaaccess $XPANAME` 
   if  [ "$a" = "yes" ] ; then
      X=500
      sleep 3
   else 
      sleep 1 
      X=`expr $X + 1`
   fi
done

if [ $X -eq 500 ] ; then

   echo " -- TEST QUOTES and PARENTHESES" >>  $OUTFILE 

   xpaset -p $XPANAME open dmlist 
   sleep 10 

   xpaset -p $XPANAME set infile "$INDIR/acis_l1_edit.fits[sky=field()][cols ra,dec]" >> $OUTFILE

   sleep 10
   echo " retrieving filename " >> $OUTFILE
   xpaget $XPANAME file >>  $OUTFILE
   echo "" >> $OUTFILE

   xpaset -p $XPANAME set opt cols  >>  $OUTFILE
   xpaset -p $XPANAME set verbose 0 >>  $OUTFILE
   xpaset -p  $XPANAME set outfile ""  >>  $OUTFILE
   echo " infile:" >> $OUTFILE
   xpaget $XPANAME get infile >>  $OUTFILE
   echo "" >> $OUTFILE
   echo " outfile:" >> $OUTFILE
   xpaget $XPANAME get outfile >>  $OUTFILE
   echo "" >> $OUTFILE
   echo " opt:" >> $OUTFILE
   xpaget $XPANAME get opt >>  $OUTFILE
   echo "" >> $OUTFILE
   echo " verbose:" >> $OUTFILE
   xpaget $XPANAME get verbose >>  $OUTFILE
   echo "" >> $OUTFILE

   echo "- check if taskmonitor already running?" >> $OUTFILE
   
   bb=`xpaaccess taskmonitor`
   if [ "$bb" == "yes" ]; then
      xpaget $XPANAME quit
      echo "$TESTID FAILED - please ensure taskmonitor is not running"
      echo "$TESTID FAILED - please ensure taskmonitor is not running" >> $OUTFILE
      exit 1 
   fi
   
   echo " run dmlist " >>  $OUTFILE
   xpaget $XPANAME run
   sleep 5
   
   # wait upto five minutes before exiting
   while [ $X -le 300 ]
   do
      a=`xpaaccess taskmonitor` 
      if  [ "$a" = "yes" ] ; then
         X=500
         sleep 3
      else 
         sleep 1 
         X=`expr $X + 1`
      fi
   done
   
   if [ $X -eq 500 ] ; then
     echo "retrieve taskmonitor status value " >> $OUTFILE
     xpaget taskmonitor status >>  $OUTFILE
     echo "" >> $OUTFILE
     xpaget taskmonitor quit
   else
     echo "FAILED: peg application 2 timed out" >> $OUTFILE
   fi
else
   echo "FAILED: peg application time out at launch" >> ${OUTFILE}
fi
    xpaget $XPANAME quit

# remove variable portion of path from log file
sed -e "s;${INDIR};;g"  ${OUTFILE} > ${OUTFILE2}

#
# Run checks on the output files.
#
passState='FAIL'

echo " $TESTID LOG " > ${LOGFILENAME}
echo "diff ${OUTFILE2} ${SAVDIR}/${TESTID}.txt >> ${DIFF_FILE} " >> ${LOGFILENAME}
echo

diff ${OUTFILE2} ${SAVDIR}'/'${TESTID}'.txt' 2>&1 >> ${DIFF_FILE}
DIFF_RETURN_RESULT=$?

# For testing, DIFF_TRUE_RESULT is set an opposite value to ensure failure
# on the if condition test below.

if test $FORCE_FAIL = 1
then
  case $DIFF_RETURN_RESULT in 
    0) DIFF_TRUE_RESULT=1
       ;;
    *) DIFF_TRUE_RESULT=0
  esac
fi

if test $DIFF_RETURN_RESULT = $DIFF_TRUE_RESULT 
then
   passState='PASS'
   echo $TESTID 
else
   echo "$TESTID ${OUTFILE} ${OUTFILE2} ${SAVDIR}/${TESTID}.txt ${LOGFILENAME} ${DIFF_FILE}"

fi

echo
echo "$TESTID : $passState"

if test "$passState" != "PASS"
then
  echo "log file in " ${LOGFILENAME}
fi

#
# Record and output the elapsed time. 
#
stopTime=`perl -e 'use Time::HiRes qw(time); print time'`
ellapsed=`echo "scale=4; $stopTime-$startTime"|bc`

echo
echo
echo " #*** Test Execution Time ("$TESTID.t"): " `convertsecstohhmmss $ellapsed`
echo

#
# Exit condition
#
if test "$passState" = "PASS"
then
  if echo "${OUTDIR}" | grep "$TESTID" >/dev/null
  then     
     rm ${OUTDIR}/param/dmlist.par
     rmdir ${OUTDIR}/param
     rm ${OUTFILE}
     rm ${OUTFILE2}
     rm ${DIFF_FILE}
     rm ${LOGFILENAME}
     rmdir ${OUTDIR}
     echo ""
  fi
fi

#End of for loop through all peg tests.
