// 
//  Copyright (C) 2002-2003,2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegApp.hh"
#include "PegCmdButtons.hh"
#include "PegParData.hh"
#include "PegUtils.hh"

#include "Parameter.hh"
#include "AhelpInterface.hh"

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
PegCmdButtons::PegCmdButtons()
{
   int pos = 0;

   // create file name label
   _label = "No file specified...";
   _fileLbl = gtk_label_new(_label.c_str());

   // create the buttons in toolbar
   _toolbar = gtk_toolbar_new ();

   GtkTooltips* tips = NULL;
   if (theApp)  tips = theApp->get_tips_group();
      else      tips = gtk_tooltips_new();

   gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_TEXT);

   if (_runExitBtn = gtk_tool_button_new(NULL, "Run & Exit"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _runExitBtn, pos++);
      g_signal_connect(_runExitBtn, "clicked", G_CALLBACK(RunExitCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_runExitBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_runExitBtn), GTK_TOOLTIPS(tips),
         "Execute the task and exit the parameter editor", "Run & Exit Button");
   }
   if (_runBtn = gtk_tool_button_new(NULL, "Run"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _runBtn, pos++);
      g_signal_connect(_runBtn, "clicked", G_CALLBACK(RunCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_runBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_runBtn), GTK_TOOLTIPS(tips),
         "Execute the task but do not exit the parameter editor", "Run Button");
   }
   if (_saveBtn = gtk_tool_button_new(NULL, "Save"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _saveBtn, pos++);
      g_signal_connect(_saveBtn, "clicked", G_CALLBACK(SaveCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_saveBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_saveBtn), GTK_TOOLTIPS(tips),
         "Save the updated parameter values", "Save Button");
   }
   if (_punlearnBtn = gtk_tool_button_new(NULL, "Punlearn"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _punlearnBtn, pos++);
      g_signal_connect(_punlearnBtn, "clicked", G_CALLBACK(PunlearnCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_punlearnBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_punlearnBtn), GTK_TOOLTIPS(tips),
         "Reset parameter entries back to their default values", "Punlearn Button");
   }
   if (_helpBtn = gtk_tool_button_new(NULL, "Tool Help"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _helpBtn, pos++);
      g_signal_connect(_helpBtn, "clicked", G_CALLBACK(HelpCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_helpBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_helpBtn), GTK_TOOLTIPS(tips),
         "Get help on the tool whose parameter file is being edited",
	 "Tool Help Button");
   }
   if (_quitBtn = gtk_tool_button_new(NULL, "Quit"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _quitBtn, pos++);
      g_signal_connect(_quitBtn, "clicked", G_CALLBACK(QuitCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_quitBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_quitBtn), GTK_TOOLTIPS(tips),
	 "Exit the parameter editor", "Quit Button");
   }

   // set up command line options field 
   GtkWidget *lab = gtk_label_new("Command Line Arguments ");
   _cmdline = gtk_entry_new();
   gtk_tooltips_set_tip(tips, _cmdline ,
      "Commandline options and flags for tool execution", NULL);

   // place widgets
   _layout = gtk_table_new(3, 10, true); 
   gtk_table_attach_defaults(GTK_TABLE(_layout), _fileLbl, 1,9,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_layout), _toolbar, 1,9,1,2);
   GtkWidget* _hbox = gtk_hbox_new(false, 1); 
   gtk_box_pack_start(GTK_BOX(_hbox), lab, 0, 0, 0);
   gtk_box_pack_start(GTK_BOX(_hbox), _cmdline, 1, 1, 0);
   gtk_table_attach_defaults(GTK_TABLE(_layout), _hbox, 1,9,2,3);
//   gtk_table_attach_defaults(GTK_TABLE(_layout), lab, 1,4,5,6);
//   gtk_table_attach_defaults(GTK_TABLE(_layout), _cmdline, 4,9,5,6);
   
   // Set sensitivity of buttons 
   disableRunExitBtn();
   disableRunBtn();
   disableSaveBtn();
   disablePunlearnBtn();
   disableHelpBtn();
   enableQuitBtn();
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
PegCmdButtons::~PegCmdButtons()
{
  gtk_widget_destroy(_toolbar);
  gtk_widget_destroy(_cmdline);
  gtk_widget_destroy(_fileLbl);
  gtk_widget_destroy(_layout);
}


// --------------------------------------------------------
// Execute the tool corresponding to the current param file
// --------------------------------------------------------
int PegCmdButtons::runCommand()
{
   if (thePData->Validate())
   {
      theApp->status_message("Unable to run... one or more fields contains an invalid value.");
      return 1; 
   }
   string filename = theApp->GetFilename();
   string launchCmd = "taskmonitor '";

   if (filename.length())
   {
      string msg = "executing tool- ";
      string cmd = _file_get_root_name(filename);
      cmd.append(" ");
      cmd.append(getCmdline());
      if (thePData)
      {
         Parameter* par = thePData->GetParameter();
         cmd.append(" ");
         cmd.append(par->GetValues());
         cmd.append(" ");
      }
      msg.append(cmd);
      cmd.append("' &");
      launchCmd.append(cmd);
      theApp->status_message(msg);
      system(launchCmd.c_str());
      return 0; 
   }
   else
      theApp->status_message("Unable to run corresponding tool for "+filename);
   return 1; 
}


// --------------------------------------------------------
// Callback for Run & Exit button push event
// --------------------------------------------------------
int PegCmdButtons::RunExitCallback(GObject *object, gpointer user_data)
{
   PegCmdButtons* cmdBtns = (PegCmdButtons*)user_data;

   // if the command didn't run due to invalid data do not exit 
   if (cmdBtns->runCommand() != 0)  return 1; 

   delete theApp;
   exit(0);
   return 0; 
}


// --------------------------------------------------------
// Callback for Run button push event
// --------------------------------------------------------
int PegCmdButtons::RunCallback(GObject *object, gpointer user_data)
{
   PegCmdButtons* cmdBtns = (PegCmdButtons*)user_data;
   cmdBtns->runCommand();
   return 0; 
}


// --------------------------------------------------------
// Callback for Save button push event
// --------------------------------------------------------
int PegCmdButtons::SaveCallback(GObject *object, gpointer user_data)
{
   PegCmdButtons* cmdBtns = (PegCmdButtons*)user_data;
   cmdBtns->save_file();
   return 0; 
}


// --------------------------------------------------------
// Punlearn the values of the current param file
// --------------------------------------------------------
void PegCmdButtons::punlearnCommand()
{
   string filename = theApp->GetFilename();
   string cmd = "cxcwrapper -s punlearn ";

   if (filename.length())
   {
      string msg = "Resetting ";
      msg.append(filename);
      msg.append(" to its system defaults via punlearn.");
      theApp->status_message(msg);
      cmd.append(filename);
      system(cmd.c_str());
      theApp->SetFilename(filename, 0);
   }
   else  theApp->status_message(
      "Unable to perform [ PUNLEARN ] - No parameter file specified.");
}



// --------------------------------------------------------
// Callback for Punlearn button push event
// --------------------------------------------------------
int PegCmdButtons::PunlearnCallback(GObject *object, gpointer user_data)
{
   PegCmdButtons* cmdBtns = (PegCmdButtons*)user_data;
   cmdBtns->punlearnCommand();
   return 0; 
}


// --------------------------------------------------------
// Callback for Help button push event
// --------------------------------------------------------
int PegCmdButtons::HelpCallback(GObject *object, gpointer user_data)
{
   string filename = theApp->GetFilename();
   string cmd = "cxcwrapper -x ahelp ";

   if (filename.length())
   {
      string rootfile = _file_get_root_name(filename); 
      cmd.append(rootfile);
      cmd.append(" &");
      system(cmd.c_str());
   }
   else  theApp->status_message(
         "Unable to obtain corresponding tool help - No parameter file specified.");
   return 0; 
}


// --------------------------------------------------------
// Callback for Quit button push event
// --------------------------------------------------------
int PegCmdButtons::QuitCallback(GObject *object, gpointer user_data)
{
   theApp->exit_application();
   return 0; 
}


// --------------------------------------------------------
// retrieve text from commandline text field
// --------------------------------------------------------
string PegCmdButtons::getCmdline()
{
   static string txt = "";

   if (_cmdline)  txt = gtk_entry_get_text(GTK_ENTRY(_cmdline));
   return txt;
}


// --------------------------------------------------------
// Set label to indicate current (specified) filename
// --------------------------------------------------------
void PegCmdButtons::setFileLabel(string fname)
{
   int enable = 0; 

   if (_fileLbl)
   {
      _label = fname; 
      gtk_label_set_text(GTK_LABEL(_fileLbl), fname.c_str());

      // disable tool help button if help does not exist for the param file
      AhelpStrings key, tmp, cont = "";
      AhelpInterface::CXCHelpStyle style=AhelpInterface::CXCHELP_SHORT;
      key = _file_get_root_name(fname).c_str(); 
      if (GlobalAhelpInterface::GetHelp(key, cont, style, 0, tmp) == 0)
         disableHelpBtn();
      else
         enableHelpBtn();

      enable = (fname != "No file displayed...");
   }

   if (enable)
   {
      // enable all other buttons
      enableRunExitBtn();
      enableRunBtn();
      enableSaveBtn();
      enablePunlearnBtn();

      // set title & icon to list current file
      string titl = "peg : " + _file_get_root_name(fname);
      theApp->set_window_title(titl);
   }
   else
   {
      // no param file to edit- disable buttons
      disableRunExitBtn();
      disableRunBtn();
      disableSaveBtn();
      disablePunlearnBtn();

      // clear title/icon
      theApp->set_window_title("peg");
   }
}

int PegCmdButtons::save_file(string fname, bool overwrite)
{
   int err = 0;
   if (thePData)
   {
      if ((err = thePData->Validate()) == 0)
      {
	 if (fname.length())
	 {
	    if (access(fname.c_str(), R_OK) == 0 && !overwrite)
	    {
               string msg = "Error: The file " + fname + " already exists and clobber is disabled. File not saved.";
	       theApp->status_message(msg);
               err = 1; 
	    }
            else     // save as specified filename 
	       err = thePData->Write((char*)fname.c_str(), overwrite); 
	 } 
         else   err = thePData->Write();  // save to opened file
	  
	 // if write was successful, update file label 
         if (err == 0) 
	 {
	    string tmp = thePData->GetFilePath(); 
	    setFileLabel(tmp); 
	 }
      }
      else 
      {
         if (err == 1)
            theApp->status_message("Unable to save... field contains an invalid value.");
	 else 
            theApp->status_message("Unable to save... fields contains invalid values.");
      }
   }
   else
      theApp->status_message("Unable to save... no parameter file loaded.");
   return err;
}

void PegCmdButtons::disableHelpBtn()
{ 
   gtk_widget_set_sensitive(GTK_WIDGET(_helpBtn), false); 
   theApp->enable_help_on_tool(false); 
}

void PegCmdButtons::enableHelpBtn()
{ 
   gtk_widget_set_sensitive(GTK_WIDGET(_helpBtn), true); 
   theApp->enable_help_on_tool(true); 
}


