// 
//  Copyright (C) 2002,2003,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "ParamItem.hh"

#include "PegApp.hh"
#include "PegParEntry.hh"
#include "PegFloatingMenu.hh"

// static members
GdkColor PegParEntry::_errcolor = { 0, 65335, 0, 0};
string   PegParEntry::_amarkup = "%s"; 
string   PegParEntry::_hmarkup = "%s"; 
int      PegParEntry::_setColors = 1;

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
PegParEntry::PegParEntry(ParamItem *item)
{
   _entry_type = PegParEntry::NO_PARAM; 
   _txt_w   = NULL;
   _ind_txt = NULL;
   _ind_lbl = NULL;
   _ind_val = NULL;
   _enumer  = NULL; 
   _item = new ParamItem(*item); 
   if (_item)
   {
      _name = _item->GetName(); 
   }
   _pos = _item->GetPosition(); 
   _popup = NULL; 


   _entry = gtk_hbox_new(false, 10); 
   _lbl_w = gtk_label_new(NULL); 

   _ind_entry = gtk_hbox_new(false, 10); 
   _ind_txt = gtk_entry_new();
   _ind_lbl = gtk_label_new("->");
   _ind_val = gtk_entry_new();
   gtk_box_pack_start(GTK_BOX(_ind_entry), _ind_txt, true, true, 0);
   gtk_box_pack_start(GTK_BOX(_ind_entry), _ind_lbl, true, true, 0);
   gtk_box_pack_start(GTK_BOX(_ind_entry), _ind_val, true, true, 0);
   gtk_entry_set_editable(GTK_ENTRY(_ind_val), false);  

   // add event boxes so we can use RMB press for floating menu
   _lbl_box = gtk_event_box_new();
   g_signal_connect(_lbl_box, "button-press-event", G_CALLBACK(eventbox_cb), 
                    (void*)this);
   gtk_container_add(GTK_CONTAINER(_lbl_box), _lbl_w);
   _entry_box = gtk_event_box_new();
   gtk_widget_show(_lbl_box);
   g_signal_connect(_entry_box, "button-press-event", G_CALLBACK(eventbox_cb), 
                    (void*)this);
   gtk_container_add(GTK_CONTAINER(_entry_box), _entry);
   gtk_widget_show(_entry_box);
   _ind_box = gtk_event_box_new();
   g_signal_connect(_ind_box, "button-press-event", G_CALLBACK(eventbox_cb), 
                    (void*)this);
   gtk_container_add(GTK_CONTAINER(_ind_box), _ind_entry);
   gtk_widget_show(_ind_box);

   // setup the pango markup string for auto/hidden params
   reset_label(); 

   // get prompt tip group
   if (theApp)  _tip_grp = theApp->get_prompts_group();
      else       _tip_grp = gtk_tooltips_new();

}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
PegParEntry::~PegParEntry()
{
   if (_item)
   {
      delete _item; 
      _item = NULL; 
   }
   if (_txt_w)
   {
      delete _txt_w;
      _txt_w = NULL;
   }
   if (_ind_txt)
   {
      delete _ind_txt;
      _ind_txt = NULL;
   }
   if (_ind_val)
   {
      delete _ind_val;
      _ind_val = NULL;
   }
   if (_popup)
   {
      delete _popup; 
      _popup = NULL; 
   }
}


// --------------------------------------------------------
// Set value  
// --------------------------------------------------------
void PegParEntry::set(const char*)
{
}


// --------------------------------------------------------
// Get value  
// --------------------------------------------------------
char* PegParEntry::get()
{
   return NULL; 
}


// --------------------------------------------------------
// Toggle between default and indirection widget type
// --------------------------------------------------------
void PegParEntry::toggleDispType()
{
   DISP_TYPE old = _disp_type; 
   if (_disp_type == PegParEntry::INDIR)
      _disp_type = _default;
   else 
      _disp_type = PegParEntry::INDIR; 
   if (_disp_type != old)
      manage(); 
}


// --------------------------------------------------------
// Initialize entry
// --------------------------------------------------------
void PegParEntry::initEntry()
{
   if (_item)
   {
      string prmpt = _item->GetPrompt();
      if (prmpt.length())
      {
	 /*
         int dur, delay;

         if (theParameters) 
         {
            dur = theParameters->GetDuration();
            delay = theParameters->GetDelay();
            if ((dur < 1000) || (dur > 10000))  dur = 5000;
            if ((delay < 1) || (delay > 5000))  delay = 1000;
         }
         else 
         {
            dur = 5000;
            delay = 1000; 
         }
         theToolTipManager->NewToolTip(prmpt(), _entry, delay, NULL, dur); 
      */
      }
   }

   if (_entry)
   {
      gtk_widget_add_events(_entry, GDK_BUTTON_PRESS_MASK);

      g_signal_connect(_entry,"button-press-event",
                       G_CALLBACK(ButtonPressCB),(void*)this);
   }
   if (_lbl_w)
   {
      gtk_widget_add_events(_lbl_w, GDK_BUTTON_PRESS_MASK);
      g_signal_connect(_lbl_w,"button-press-event",
                       G_CALLBACK(ButtonPressCB),(void*)this);
   }
   _orig_type =_disp_type; 
   if (_disp_type == PegParEntry::ENUM) 
      _default = _disp_type; 
   else 
      _default = PegParEntry::DEFAULT; 
}


// --------------------------------------------------------
// Event Handler to handle popup menu on right button press
// --------------------------------------------------------

int PegParEntry::ButtonPressCB(GObject *object, GdkEvent *event,
   gpointer user_data)
{
   PegParEntry *dat = (PegParEntry*)user_data;
   GdkEventButton *evt = (GdkEventButton*)event;

   if (dat && evt && evt->button == 3)
   {
      dat->launchPopup(evt);
      return 1;  
   }
   return 0; 
}


// --------------------------------------------------------
// creates popup menu at event location
// --------------------------------------------------------
 void PegParEntry::launchPopup(GdkEventButton *event)
 {
    GdkEventButton* evt = (GdkEventButton *) event;
    _popup = new PegFloatingMenu(this);
    gtk_menu_popup(GTK_MENU(_popup->get_menu()), NULL, NULL, NULL, NULL, evt->button, evt->time);
 }


// --------------------------------------------------------
// display the entry's widget form and label 
// --------------------------------------------------------
void PegParEntry::manage()
{
}


// --------------------------------------------------------
// unmanage the entry's widget form and label 
// --------------------------------------------------------
void PegParEntry::unmanage() 
{
}


int PegParEntry::eventbox_cb(GObject *object, GdkEvent *event,
   gpointer user_data)
{
   GdkEventButton *evt = (GdkEventButton*)event;
   PegParEntry *dat = (PegParEntry*)user_data;
   if (evt && dat && evt->button == 3)
   {
      PegFloatingMenu* popup = new PegFloatingMenu(dat);
      gtk_menu_popup(GTK_MENU(popup->get_menu()), NULL, NULL, NULL, NULL,
                     evt->button, evt->time);
   }
   return 0; 
}



string PegParEntry::set_markup(string clr, string fmt)
{
   // sets up a format string to support normal, bold, italic, or
   // underlined text of the specified color using pango markups
   static string markup; 
   string::size_type bb, uu, ii; 
   markup = ""; 
   if ((bb = fmt.find("b")) != string::npos)  markup.append("<b>"); 
   if ((uu = fmt.find("u")) != string::npos)  markup.append("<u>"); 
   if ((ii = fmt.find("i")) != string::npos)  markup.append("<i>"); 
   if (clr.length()) 
   {
      markup.append("<span foreground=\""); 
      markup.append(clr);
      markup.append("\">%s</span>");
   }
   else 
      markup.append("%s"); 
   if (ii != string::npos)  markup.append("</i>"); 
   if (uu != string::npos)  markup.append("</u>"); 
   if (bb != string::npos)  markup.append("</b>"); 

   return markup; 
}


void PegParEntry::reset_label()
{
   if (_setColors)
   {
      if (theApp)
      {
         string tmp_clr ="", tmp_fmt ="";
         theApp->get_preference("auto.color", tmp_clr);
         theApp->get_preference("auto.markup", tmp_fmt);
         _amarkup = set_markup(tmp_clr, tmp_fmt);
         tmp_clr.clear();
         tmp_fmt.clear();
         theApp->get_preference("hidden.color", tmp_clr);
         theApp->get_preference("hidden.markup", tmp_fmt);
         _hmarkup = set_markup(tmp_clr, tmp_fmt);
      }
      _setColors = 0; 
   }
   char lab_buf[512];
   if (_item->IsAutoMode() || _item->IsQueryMode())
      sprintf(lab_buf, _amarkup.c_str(), _name.c_str());
   else
      sprintf(lab_buf, _hmarkup.c_str(), _name.c_str());
   gtk_label_set_markup (GTK_LABEL(_lbl_w), lab_buf);
}


