// 
//  Copyright (C) 2002,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PEGPARENTRYREAL_HH
#define PEGPARENTRYREAL_HH

#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "ParamItem.hh"
#include "PegParEntry.hh"

class PegParEntryReal : public PegParEntry {

  public:

    PegParEntryReal(ParamReal *item);

    void set(const char* val);
    char* get();

    double getMinValue()        { return _min; }
    double getMaxValue()        { return _max; }
    double getScale()           { return _scale; }
    void setScale(double val)   { _scale = val; }

    ParamReal* GetItem()        { return _item; }

    static int change_cb(GObject *object, gpointer user_data);
    static int ButtonPressCB(GObject *object, GdkEvent *event,
                             gpointer user_data);
    static int FocusOutCB(GObject *object, GdkEvent *event, gpointer user_data);
    static int TextEditCB(GObject *object, gpointer user_data);
    static int IndirEditCB(GObject *object, gpointer user_data);

    virtual void manage();
    virtual void set_error_color();    // set widget red to indicate err
    virtual void clear_error_color();  // clear error code

    int update_indir();
    int update_text();
    bool in_edit()  { return _inedit; }

    bool check_valid_input(string val);

    void set_adjustments(); 

    string get_lastval()   { return _lastval; }
    void set_lastval(string val)  { _lastval = val; }

  protected:
    ~PegParEntryReal();

  private:

   ParamReal* _item; 
   GtkWidget* _spin_w;
   GtkAdjustment *_adj;
   double     _min; 
   double     _max; 
   double     _scale;   // arrow button adjustment values;
   bool       _inedit;  
   string     _lastval;
    
};

#endif

