// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PEGPREFS_HH
#define PEGPREFS_HH

#include <stdio.h>
#include <string>
#include <list>


using namespace std;

class PegPrefs
{
  public:
    PegPrefs();
    ~PegPrefs();

    int load_preferences(string fname=""); 
    int save_preferences(string fname=""); 

    int get_preference(string prop, bool &val);
    int get_preference(string prop, string &val);

    bool get_tooltips()                  { return _tips; }
    void set_tooltips(bool val)          { _tips = val; }

    bool get_clobber()                   { return _clobber; }
    void set_clobber(bool val)           { _clobber = val; }

    bool get_show_hidden()               { return _showhidden; }
    void set_show_hidden(bool val)       { _showhidden = val; }

    bool get_display_prompts()           { return _prompts; }
    void set_display_prompts(bool val)   { _prompts = val; }

    string get_autocolor()               { return _autocolor; }
    int    set_autocolor(string val); 
    string get_hiddencolor()             { return _hiddencolor; }
    int    set_hiddencolor(string val); 
    string get_automarkup()              { return _automarkup; } 
    int    set_automarkup(string val); 
    string get_hiddenmarkup()            { return _hiddenmarkup; } 
    int    set_hiddenmarkup(string val); 

  protected: 
    int  parse_line(const char *in_txt, char del);  
    void initialize();             // set initial values 

    string filename;               // name of loaded preferences file

    // general 
    string  _autocolor;            // auto parameter color 
    string  _automarkup;           // markup tags for auto params 
    string  _hiddencolor;          // hidden parameter color 
    string  _hiddenmarkup;         // markup tags for auto params 
    bool    _clobber;              // overwrite existing file
    bool    _tips;                 // display tooltips 
    bool    _showhidden;           // display hidden parameters 
    bool    _prompts;              // display param prompt as tooltip 
};


#endif 
