// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegPrefsDialog.hh"
#include "PegPrefs.hh"

#include <iostream>

using namespace std;


PegPrefsDialog::PegPrefsDialog(PegPrefs *prefs, GtkTooltips* tipgroup)
{
   _prefs = prefs; 
   _tip_gp = tipgroup; 
   load_prefs();
}

PegPrefsDialog::~PegPrefsDialog()
{
}

void PegPrefsDialog::generate_form()
{
   base_w = gtk_dialog_new_with_buttons ("Peg Prefereneces", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_SAVE, GTK_RESPONSE_APPLY,
      GTK_STOCK_OK, GTK_RESPONSE_OK,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   gtk_widget_set_size_request(base_w, 300, 200);
   _scrolled = gtk_scrolled_window_new (NULL,NULL);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

   _vbox = gtk_vbox_new (FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(_vbox), 5);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(_scrolled), _vbox);

   _tips_w = gtk_check_button_new_with_label("Display Tool Tips");
   _prompts_w = gtk_check_button_new_with_label("Display Parameter Prompts");
   _showhidden_w = gtk_check_button_new_with_label("Display Hidden Parameters");
   _clobber_w = gtk_check_button_new_with_label("Overwrite existing files");

   _auto_exp = gtk_expander_new("Auto Parameter Options");
   _auto_table = gtk_table_new(4,10, false);
   gtk_container_add (GTK_CONTAINER (_auto_exp), _auto_table); 
   _auto_bold_w = gtk_check_button_new_with_label("Bold");
   _auto_italic_w = gtk_check_button_new_with_label("Italic");
   _auto_uline_w = gtk_check_button_new_with_label("Underline");
   _auto_def_w = gtk_check_button_new_with_label("Default");
   _autlab_w = gtk_label_new("Style"); 

   GtkWidget* lab = gtk_label_new("Color"); 
   GdkColor clr;
   if (_autocolor.length())
      gdk_color_parse(_autocolor.c_str(), &clr);
   else 
      gdk_color_parse("black", &clr);
   _autofg_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_autofg_w),
                             "Automatic Parameter Color"); 

   gtk_table_attach_defaults(GTK_TABLE(_auto_table), lab, 0,1, 0,1);
   gtk_table_attach_defaults(GTK_TABLE(_auto_table), _auto_def_w, 1,2,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_auto_table), _autofg_w, 2,3,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_auto_table), _autlab_w, 0,1, 3,4);
   gtk_table_attach_defaults(GTK_TABLE(_auto_table), _auto_bold_w, 1,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_auto_table), _auto_italic_w, 2,3,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_auto_table), _auto_uline_w, 3,4,3,4);
   
   lab = gtk_label_new("Color"); 
   if (_hidcolor.length())
      gdk_color_parse(_hidcolor.c_str(), &clr);
   else 
      gdk_color_parse("black", &clr);
   _hidfg_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_hidfg_w),
                             "Hidden Parameter Color"); 
   _hid_exp = gtk_expander_new("Hidden Parameter Options");
   _hid_table = gtk_table_new(4,17, false);
   _hidlab_w = gtk_label_new("Style"); 
   gtk_container_add(GTK_CONTAINER(_hid_exp), _hid_table); 

   _hid_bold_w = gtk_check_button_new_with_label("Bold");
   _hid_italic_w = gtk_check_button_new_with_label("Italic");
   _hid_uline_w = gtk_check_button_new_with_label("Underline");
   _hid_def_w = gtk_check_button_new_with_label("Default");
   
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), lab, 0,1,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), _hid_def_w, 1,2,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), _hidfg_w, 2,3,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), _hidlab_w, 0,1,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), _hid_bold_w, 1,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), _hid_italic_w, 2,3,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_hid_table), _hid_uline_w, 3,4,3,4);

   gtk_box_pack_start(GTK_BOX(_vbox), _tips_w, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _prompts_w, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _showhidden_w, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _clobber_w, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _auto_exp, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _hid_exp, FALSE, FALSE, 0);
   
   load_dialog_prefs(); 
   set_color_buttons_sensitive();

   g_signal_connect(_hid_def_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);
   g_signal_connect(_auto_def_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   if (_tip_gp)
   {
      gtk_tooltips_set_tip(_tip_gp, _tips_w, 
         "Display tool tips when cursor over widgets", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _prompts_w, 
         "Display parameters prompts like tool tips", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _showhidden_w, 
         "Display hidden parameters in the gui", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _clobber_w,
         "If specified file already exists, overwrite it", NULL);
      gtk_tooltips_set_tip(_tip_gp, _auto_def_w,
         "Use the default theme's color for auto parameter labels", NULL);
      gtk_tooltips_set_tip(_tip_gp, _auto_bold_w,
         "Display auto parameters using a bold face", NULL);
      gtk_tooltips_set_tip(_tip_gp, _auto_italic_w,
         "Display auto parameters using italic style", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _auto_uline_w,  
         "Display auto parameters using an underline", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _hid_def_w,
         "Use the default theme's color for hidden parameter labels", NULL);
      gtk_tooltips_set_tip(_tip_gp, _hid_bold_w,
         "Display hidden parameter labels using a bold face", NULL);
      gtk_tooltips_set_tip(_tip_gp, _hid_italic_w,
         "Display hidden parameter labels using italic style", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _hid_uline_w,  
         "Display hidden parameters using an underline", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _autofg_w,
         "Specify a custom color to indicate auto parameters", NULL); 
      gtk_tooltips_set_tip(_tip_gp, _hidfg_w,
         "Specify a custom color to indicate hidden parameters", NULL); 
   }

   show();
}

int PegPrefsDialog::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}


void PegPrefsDialog::doit()
{
   generate_form(); 

   int resp = gtk_dialog_run(GTK_DIALOG(base_w)); 

   if (resp == GTK_RESPONSE_APPLY)
   {
      // save to file 
      save_prefs();
   }
   else if (resp == GTK_RESPONSE_OK)
   {
      // update local preferences 
      commit_prefs();
   }
   gtk_widget_destroy(base_w);
}

void PegPrefsDialog::auto_prefs_to_strings()
{
   if (_autodef)   _autocolor = "";  
   _automarkup = "";
   if (_autobold)       _automarkup.append("b");
   if (_autounderline)  _automarkup.append("u");
   if (_autoitalic)     _automarkup.append("i");
}

void PegPrefsDialog::auto_prefs_from_strings()
{
   _autodef = (_autocolor.length() == 0);
   _autobold = ((_automarkup.find("b") != string::npos) ||
                (_automarkup.find("B") != string::npos));   
   _autounderline = ((_automarkup.find("u") != string::npos) || 
                     (_automarkup.find("U") != string::npos));
   _autoitalic = ((_automarkup.find("i") != string::npos) ||
                  (_automarkup.find("I") != string::npos));   
}

void PegPrefsDialog::hidden_prefs_to_strings()
{
   if (_hiddef)   _hidcolor = "";  
   _hidmarkup = "";
   if (_hidbold)  _hidmarkup.append("b");
   if (_hidunderline)  _hidmarkup.append("u");
   if (_hiditalic)  _hidmarkup.append("i");
}

void PegPrefsDialog::hidden_prefs_from_strings()
{
   _hiddef = (_hidcolor.length() == 0);
   _hidbold = ((_hidmarkup.find("b") != string::npos) || 
               (_hidmarkup.find("B") != string::npos)); 
   _hidunderline = ((_hidmarkup.find("u") != string::npos) || 
                    (_hidmarkup.find("U") != string::npos));
   _hiditalic = ((_hidmarkup.find("i") != string::npos) ||
                 (_hidmarkup.find("I") != string::npos));
}


void PegPrefsDialog::load_prefs()
{
   if (!_prefs) return; 
   _tips =_prefs->get_tooltips(); 
   _prompts = _prefs->get_display_prompts(); 
   _showhidden = _prefs->get_show_hidden(); 
   _clobber = _prefs->get_clobber();
   _autocolor = _prefs->get_autocolor();
   _hidcolor = _prefs->get_hiddencolor();
   _automarkup = _prefs->get_automarkup();
   _hidmarkup = _prefs->get_hiddenmarkup();

   auto_prefs_from_strings();
   hidden_prefs_from_strings(); 
}

void PegPrefsDialog::commit_prefs()
{
   retrieve_dialog_prefs(); 

   if (!_prefs) return; 

   auto_prefs_to_strings(); 
   hidden_prefs_to_strings(); 
   _prefs->set_tooltips(_tips);
   _prefs->set_display_prompts(_prompts); 
   _prefs->set_show_hidden(_showhidden); 
   _prefs->set_clobber(_clobber);
   _prefs->set_autocolor(_autocolor);
   _prefs->set_automarkup(_automarkup); 
   _prefs->set_hiddencolor(_hidcolor);
   _prefs->set_hiddenmarkup(_hidmarkup); 
}

void PegPrefsDialog::save_prefs()
{
   if (!_prefs) return; 
   commit_prefs();
   _prefs->save_preferences(); 
}

void PegPrefsDialog::load_dialog_prefs()
{
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tips_w), _tips);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_prompts_w), _prompts);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_showhidden_w), _showhidden);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_clobber_w), _clobber);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_auto_bold_w), _autobold);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_auto_italic_w), _autoitalic);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_auto_uline_w), _autounderline);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_auto_def_w), _autodef);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hid_bold_w), _hidbold);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hid_italic_w), _hiditalic);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hid_uline_w), _hidunderline);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hid_def_w), _hiddef);

   GdkColor clr;
   if (_autocolor.length())
   {
      gdk_color_parse(_autocolor.c_str(), &clr);
      gtk_color_button_set_color(GTK_COLOR_BUTTON(_autofg_w), &clr);
   }
   if (_hidcolor.length())
   {
      gdk_color_parse(_hidcolor.c_str(), &clr);
      gtk_color_button_set_color(GTK_COLOR_BUTTON(_hidfg_w), &clr);
   }
}

void PegPrefsDialog::retrieve_dialog_prefs()
{
   _tips = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tips_w));
   _prompts = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_prompts_w));
   _showhidden = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_showhidden_w));
   _clobber = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_clobber_w));
   _autodef = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_auto_def_w));
   _autobold = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_auto_bold_w));
   _autoitalic = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_auto_italic_w));
   _autounderline = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_auto_uline_w));
   _hiddef = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hid_def_w));
   _hidbold = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hid_bold_w));
   _hiditalic = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hid_italic_w));
   _hidunderline = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hid_uline_w));

   GdkColor clr;
   gtk_color_button_get_color(GTK_COLOR_BUTTON(_autofg_w), &clr);
   char hexconv[32];
   int rr, gg, bb;
   if (!gdk_color_equal(&clr, &_prevac))
   {
      // since we're converting to rrggbb- only update if the original changed
      if ((rr = clr.red) > 0) rr--;
      if ((gg = clr.green) > 0) gg--;
      if ((bb = clr.blue) > 0) bb--;
      sprintf(hexconv, "#%02x%02x%02x", rr/256, gg/256, bb/256); 
      _autocolor = hexconv; 
   }
   gtk_color_button_get_color(GTK_COLOR_BUTTON(_hidfg_w), &clr);
   if (!gdk_color_equal(&clr, &_prevhc))
   {
      // since we're converting to rrggbb- only update if the original changed
      if ((rr = clr.red) > 0) rr--;
      if ((gg = clr.green) > 0) gg--;
      if ((bb = clr.blue) > 0) bb--;
      sprintf(hexconv, "#%02x%02x%02x", rr/256, gg/256, bb/256); 
      _hidcolor = hexconv; 
   }
}


int PegPrefsDialog::toggled_value_callback(GObject *object, gpointer user_data)
{
   PegPrefsDialog* dlg = (PegPrefsDialog*)user_data;
   if (!(dlg))  return 1;
   dlg->set_color_buttons_sensitive();
   return 0;
}

void PegPrefsDialog::set_color_buttons_sensitive()
{
   _autodef = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_auto_def_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_autofg_w), !_autodef);
   _hiddef = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hid_def_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_hidfg_w), !_hiddef);
}

