// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PEGPREFSDIALOG_HH
#define PEGPREFSDIALOG_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <gdk/gdk.h>
#include <string>

using namespace std;

class PegPrefs;

class PegPrefsDialog
{
  public:
    PegPrefsDialog(PegPrefs *prefs, GtkTooltips* tipgroup);
    virtual ~PegPrefsDialog();
    void doit(); 

    static int toggled_value_callback(GObject *object, gpointer user_data);
    void set_color_buttons_sensitive();

  protected:
    virtual int show();
    void generate_form(); 

    GtkWidget* base_w; 
    GtkWidget* _scrolled; 
    GtkWidget* _vbox; 
    GtkWidget* _tips_w;
    GtkWidget* _clobber_w;
    GtkWidget* _prompts_w; 
    GtkWidget* _showhidden_w; 

    GtkWidget* _auto_exp;
    GtkWidget* _auto_table;
    GtkWidget* _auto_bold_w; 
    GtkWidget* _auto_italic_w; 
    GtkWidget* _auto_uline_w; 
    GtkWidget* _auto_def_w; 

    GtkWidget* _hid_exp;
    GtkWidget* _hid_table;
    GtkWidget* _hid_bold_w; 
    GtkWidget* _hid_italic_w; 
    GtkWidget* _hid_uline_w; 
    GtkWidget* _hid_def_w; 

    GtkWidget* _autofg_w; 
    GtkWidget* _hidfg_w;

    GtkWidget* _autlab_w;
    GtkWidget* _hidlab_w;

    void load_prefs(); 
    void load_dialog_prefs(); 
    void retrieve_dialog_prefs();
    void commit_prefs(); 
    void save_prefs(); 

  private:
    void auto_prefs_to_strings();
    void hidden_prefs_to_strings();
    void auto_prefs_from_strings();
    void hidden_prefs_from_strings();

    PegPrefs* _prefs; 

    string _autocolor;        // custom color for auto params
    string _hidcolor;         // custom color for hidden params 
    string _automarkup;       // pango markup flags for auto params 
    string _hidmarkup;        // pango markup flags for hidden params 
    bool   _tips;             // display tool tips 
    bool   _clobber;          // overwrite existing file 
    bool   _showhidden;       // display hidden parameter entries 
    bool   _prompts;          // display prompts as tool tips 
    bool   _autobold;         // bold face for auto params  
    bool   _autoitalic;       // italic style for auto params 
    bool   _autodef;          // default theme color for auto params 
    bool   _autounderline;    // underline auto params  
    bool   _hidbold;          // bold face for hidden params  
    bool   _hiditalic;        // italic style for hidden params 
    bool   _hidunderline;     // underline hidden params  
    bool   _hiddef;           // default theme color for hidden params 
    GdkColor _prevac;         // previous foreground select color
    GdkColor _prevhc;         // previous background select color
    GtkTooltips* _tip_gp;     // tools tips group
};

#endif 
