// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegApp.hh"
#include "PegSaveAsDialog.hh"

#include <fstream>
#include <iostream>
#include <unistd.h>

using namespace std;

string PegSaveAsDialog::defpath = ""; 

PegSaveAsDialog::PegSaveAsDialog(string usepath)
{
   if (usepath.length())   defpath = usepath; 
}

PegSaveAsDialog::~PegSaveAsDialog()
{
   if (dialog)   gtk_widget_destroy (dialog);
}

int PegSaveAsDialog::doit()
{
   int rval = 0; 
   dialog = gtk_file_chooser_dialog_new("Save File As...",
                                         GTK_WINDOW(NULL),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         NULL);
   if (defpath.length())
      gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), 
                                          defpath.c_str());
   // add basic filter support
   GtkFileFilter *filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*");
   gtk_file_filter_set_name(filt,"All Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.par");
   gtk_file_filter_set_name(filt,"Parameter Files (*.par)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      gchar * filename = (char*)gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)
      {
         string fn = filename;
	 // check if file exists and has write access
         if (access(filename, F_OK) == 0 && access(filename, W_OK) == -1)
         {
            string msg = "Error: Improper permissions to write to " + fn + ". File not saved.";
            theApp->status_message(msg);
            rval = 1;
         }
         else
         {
	    // verify that the path, if provided, exists
	    string pth = filename;
	    pth = pth.substr(0, pth.find_last_of("/"));
	    if (pth.length())
	    {
	       if (access(pth.c_str(), F_OK) != 0)
               {
                  theApp->status_message("Error: The specified path " + pth + " does not exist. File not saved.");
                  rval = 1;
               }
               else if (access(pth.c_str(), W_OK) != 0)
               {
                  theApp->status_message("Error: Improper permissions to write todirectory " + pth + ". File not saved.");
		  rval = 1;
               }
            }
	 }
         string newpath = fn;
	 unsigned int delim = newpath.find_last_of('/');

	 if (_check_is_directory(newpath))
         {
            if (delim + 1 != newpath.length()) newpath.append("/");
         }
         else if (delim != string::npos)
            newpath = newpath.substr(0, delim+1);
         defpath = newpath;
         g_free(filename); 
      }
      if (rval == 0)
      {
         // if save to new file successful, use the new file as base
         if ((rval = theApp->save_file(filename)) == 0)
             theApp->SetFilename(filename);
      }
   }
   else 
      rval = -1;  // we return 0 for a successful save, so return non zero here
   return rval;
}

