#!/bin/sh

# Testing Script for prism.
# usage:  prism_test00[1-6].t 
# To test fail condition, usage: prism_test00[1-6].t force_fail
# SLEEP_DURATION operation time is 10.  The variable is set to a small number (0.1) for testing.

TESTPATH="$( cd "$( dirname "$0" )" && pwd )" 
source "$TESTPATH/prism_common.sh"
source "$TESTPATH/prism_tests.sh"

TESTID="prism_test001" 

FORCE_FAIL=0

if test "$1" = "force_fail"
then
  FORCE_FAIL=1
fi

DIFF_TRUE_RESULT=0

export TESTID

# In function set_test_env, set OUTDIR, OUTFILE, OUTFILE2, SAVDIR
#                               LOGNAMEFILE, INDIR, DIFF_FILE

set_test_env

echo 
echo " #*** Test Begin: $TESTID.t"
echo -e "\n"
echo "$TESTID regression"
echo -e "\n"
echo "running" $TESTID

#
# Run tests
#
echo " $TESTID LOG " > ${OUTFILE}
echo "" >> ${OUTFILE}

startTime=`perl -e 'use Time::HiRes qw(time); print time'`

prism -x prism &
X=0 
sleep $SLEEP_DURATION 

# wait upto five minutes before exiting

while [ $X -le 300 ]
do
   a=`xpaaccess prism` 
   if  [ "$a" = "yes" ] ; then
      X=500
   else 
      sleep 1 
      X=`expr $X + 1`
   fi

if [ $X -eq 500 ] ; then

   `$TESTID`

else
   echo "FAILED: prism application time out at launch" >> ${OUTFILE}
fi

# remove variable portion of path from log file
sed -e "s;${INDIR};;g"  ${OUTFILE} > ${OUTFILE2}

#
# Run checks on the output files.
#
passState='FAIL'

echo " $TESTID LOG " > ${LOGFILENAME}
echo "diff ${OUTFILE2} ${SAVDIR}/${TESTID}.txt >> ${DIFF_FILE} " >> ${LOGFILENAME}
echo

diff ${OUTFILE2} ${SAVDIR}'/'${TESTID}'.txt' 2>&1 >> ${DIFF_FILE}
DIFF_RETURN_RESULT=$?

# For testing, DIFF_TRUE_RESULT is set an opposite value to ensure failure
# on the if condition test below.

if test $FORCE_FAIL = 1
then
  case $DIFF_RETURN_RESULT in 
    0) DIFF_TRUE_RESULT=1
       ;;
    *) DIFF_TRUE_RESULT=0
  esac
fi

if test $DIFF_RETURN_RESULT = $DIFF_TRUE_RESULT 
then
   passState='PASS'
   echo $TESTID 
else
   echo "$TESTID ${OUTFILE} ${OUTFILE2} ${SAVDIR}/${TESTID}.txt ${LOGFILENAME} ${DIFF_FILE}"

fi

echo
echo "$TESTID : $passState"

if test "$passState" != "PASS"
then
  echo "log file in " ${LOGFILENAME}
fi

#
# Record and output the elapsed time. 
#
stopTime=`perl -e 'use Time::HiRes qw(time); print time'`
ellapsed=`echo "scale=4; $stopTime-$startTime"|bc`

echo
echo
echo " #*** Test Execution Time ("$TESTID.t"): " `convertsecstohhmmss $ellapsed`
echo

#
# Exit condition
#
if test "$passState" = "PASS"
then
  if echo "${OUTDIR}" | grep "$TESTID" >/dev/null
  then
     rm ${OUTFILE}
     rm ${OUTFILE2}
     rm ${DIFF_FILE}
     rm ${LOGFILENAME}
     rmdir ${OUTDIR}
     echo ""
  fi
fi

done
#End of for loop through all prism tests.
