// 
//  Copyright (C) 2007,2010-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMAPP_HH
#define PRISMAPP_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

#include <Python.h>

class PrismMenu; 
class PrismStatusArea;
class PrismFileList;
class PrismPrefs;
class PrismPrefsDialog; 
class CiaoGuiIPCServer;
class CiaoGuiIPCMessage; 

using namespace std;

class PrismApp
{
  public:
    PrismApp();
    ~PrismApp();

    static void exit_cb();

    int load_file(string filename, int new_tab=0, int editable=0);
    int close_tab();

    int status_message(string msg, int type=0); 
    string get_status_messages(); 
    int  get_status_msg_cnt(); 
    void initialize(string toolmenu="",string xpaname="",char** filenames=NULL);
    void change_error_color(string cname);
    void set_error_bold(bool val);
    void set_error_italic(bool val);
    void set_error_color(bool val); 
    int quit();

    void show_tabs();
    void hide_tabs();
    bool tabs_visible(); 
    int  get_num_tabs(); 
    void reposition_tabs(int val);
    int  get_tab_position(); 
    void launch_preferences_dialog(); 
    int  set_current_tab(int val); 
    int  get_current_tab(); 

    bool use_image_dialog()  { return _img_dialog; }
    bool can_image()         { return (_found_ds9 && _found_xpa); }
    bool can_plot()          { return _found_chips;   }

    int visualize_current_tab(int type, int dialog=1); 
    int select_in_current_tab(int val, int as_col);  
    int reload_current_tab(int ext = 1, long row = 1);
    int lock_current_tab(int mode);
    int get_current_mode(); 

    int get_preference(string pref, bool &val);
    int get_preference(string pref, long &val); 
    int get_preference(string pref, string &val); 
    bool check_column_expansion(string colname, int vdim, int adim); 
    string get_data_format(string cname, string ename, int typ);

    int print_current_plot(); 
    int connect_plotter(string id); 
    PyObject *call_py_functions(PyObject *pModule, const string & func_name,
                                const string &format="", ...);

    void set_modified(); 

    GtkTooltips* get_tips_group()  { return _tips; }
    void  reset_tips(); 

    string get_filename(int tab=-1, int filter=0); 
    string get_chipsId()           { return _chipsId; }
    string get_xpaname()           { return _xpaname; }

    static string get_current_filename();
    static void set_status_message(string val); 

  protected: 
    void check_dependencies();    // check for chips, ds9, xpaset
    void cleanup();               // free memory, close resources 

    GtkWidget *window;
    GtkWidget *vbox;
    GtkWidget *vpane;
    GtkTooltips *_tips;            // tool tips group

    PrismFileList *_fileList; 
    PrismMenu *menu;
    PrismStatusArea *statusArea;
    PrismPrefs *_prefs; 
    PrismPrefsDialog *_prefsDialog;

    int  def_height;               // default height 
    int  def_width;                // default width  
    int  _init;                    // have we initialized yet? 

    bool _showtips;                // show tool tips 
    bool _img_dialog;              // datatool bar img button invokes dialog?

    bool _found_ds9;               // ds9 executable exists 
    bool _found_xpa;               // xpaset executable exists 
    bool _found_chips;             // chipsServer executable exists 

    PyObject *_pyModule;           // python module
    string _chipsId;               // id of chips server
    string _xpaname;               // xpa access name

    static CiaoGuiIPCMessage* VizMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* FileMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* StatusMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* TabMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* SelectMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* GetValueMessageCB(CiaoGuiIPCMessage&);

    void   add_ipc();
    CiaoGuiIPCServer* ipc;

};

extern PrismApp *theApp;

#endif 
