// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismAppendRowCmd.hh"
#include "PrismDataTable.hh"
#include "PrismApp.hh"
#include <iostream>
#include <sstream>

using namespace std;

PrismAppendRowCmd::PrismAppendRowCmd():PrismBaseObj()
{
   _toggle = 0;
}

PrismAppendRowCmd::~PrismAppendRowCmd()
{
}

void PrismAppendRowCmd::create_dialog()
{
   base_w = gtk_dialog_new_with_buttons("Append Row to Table", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(5,6, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), tab);

   stringstream ss;
   string rowloc;

   _rowmin = 1;  
   _rowmax = _table->get_nrows();
   _rowcurr = 1; 
   long cp = _table->get_current_page(); 
   if (cp>0)  _rowcurr += (cp-1) * _table->get_pagesize(); 

   ss << "   (" << _rowcurr << '/' << _rowmax << ')';
   ss >> rowloc; 
   ss.clear();

   GtkWidget* locrowlab = gtk_label_new(rowloc.c_str());
   GtkWidget* labtog = gtk_label_new("Initialization:");
   _rowtog_w = gtk_radio_button_new_with_label(NULL, "Use Existing Row ");
   _zerotog_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_rowtog_w), "Zero Out");
   if (_toggle) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_zerotog_w), true);
      else      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_rowtog_w), true);
   g_signal_connect(_zerotog_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   GtkWidget* labrow = gtk_label_new("Row #");
   GtkAdjustment *row_adj = (GtkAdjustment *) 
      gtk_adjustment_new (_rowcurr, _rowmin, _rowmax, 10.0, 5.0, 0.0);
   _rowspin_w   = gtk_spin_button_new(row_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_rowspin_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_rowspin_w),GTK_UPDATE_IF_VALID);
   g_signal_connect(_rowspin_w, "value-changed", G_CALLBACK(rowspin_value_callback),
                    (void*)this);
   _rowscale_w = gtk_hscale_new(row_adj);
   g_signal_connect(_rowscale_w,"value-changed",G_CALLBACK(rowscale_value_callback),
                    (void*)this);
   gtk_scale_set_draw_value(GTK_SCALE(_rowscale_w), 0);

   gtk_table_attach_defaults(GTK_TABLE(tab), labtog, 0,2,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), _rowtog_w, 2,4,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), _zerotog_w, 4,6,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), labrow, 0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _rowspin_w, 2,4,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), locrowlab, 4,6,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _rowscale_w, 1,5,3,4);
   set_sensitive(); 
   show(); 
}

int PrismAppendRowCmd::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismAppendRowCmd::doit(PrismDataTable* table)
{
   int rv  = 0; 

   if (!theApp)             return 1;   // app doesn't exist
   if (!(_table = table))   return 2;    // table doesn't exist

   create_dialog();
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
      if ((_toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_zerotog_w))))
         rv= _table->append_row(0);
      else
      {
	 long cr = (long)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_rowspin_w)); 
	 rv = _table->load_row((long)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_rowspin_w))); 
         if (!rv)  rv= _table->append_row(cr);
      }
   }
   gtk_widget_destroy(base_w);
   return rv;
}

int PrismAppendRowCmd::set_rowscale_value(long val)
{
   if ((val < _rowmin) || (val > _rowmax)) return 1; 
   if (val != (long) gtk_range_get_value(GTK_RANGE(_rowscale_w)))
      gtk_range_set_value(GTK_RANGE(_rowscale_w), val); 
   return 0; 
}

int PrismAppendRowCmd::set_rowspin_value(long val)
{
   if ((val < _rowmin) || (val > _rowmax)) return 1; 
   if (val != (long) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_rowspin_w)))
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(_rowspin_w), val);
   return 0; 
}


int PrismAppendRowCmd::rowspin_value_callback(GObject *object, gpointer user_data)
{
   GtkSpinButton *ent = GTK_SPIN_BUTTON(object);
   PrismAppendRowCmd* cmd = (PrismAppendRowCmd*)user_data;

   if (!(ent&&cmd))  return 1;
   long val = (long)gtk_spin_button_get_value(ent);
   return cmd->set_rowscale_value(val); 
}

int PrismAppendRowCmd::rowscale_value_callback(GObject *object, gpointer user_data)
{
   GtkScale *ent = GTK_SCALE(object);
   PrismAppendRowCmd* cmd = (PrismAppendRowCmd*)user_data;

   if (!(ent&&cmd))  return 1;
   long val = (long) gtk_range_get_value(GTK_RANGE(ent));
   return cmd->set_rowspin_value(val); 
}

int PrismAppendRowCmd::toggled_value_callback(GObject *object, gpointer user_data)
{
   PrismAppendRowCmd* cmd = (PrismAppendRowCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->set_sensitive(); 
   return 0;
}

void PrismAppendRowCmd::set_sensitive()
{
   _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_zerotog_w));

   gtk_widget_set_sensitive(GTK_WIDGET(_rowscale_w), !_toggle); 
   gtk_widget_set_sensitive(GTK_WIDGET(_rowspin_w), !_toggle); 
}


