// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismBackupDialog.hh"
#include "PrismApp.hh"
#include "PrismUtils.hh"

#include <fstream>
#include <iostream>

using namespace std;

PrismBackupDialog::PrismBackupDialog(string tname, string fname):PrismBaseObj()
{
   _title = tname; 
   _filename = fname; 
}

PrismBackupDialog::~PrismBackupDialog()
{
}

void PrismBackupDialog::generate_form()
{
   string ss; 

   base_w = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT, 
      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
      "Editing the file will irreversibly alter the file. Click 'Edit Copy' to edit a new copy of the file. Click 'Edit Original' to edit the current file. Click 'Cancel' to cancel...");
   GtkWidget *wid = gtk_dialog_add_button(GTK_DIALOG(base_w),
                    "Edit Copy", GTK_RESPONSE_APPLY);
   GtkWidget *img = gtk_image_new_from_stock(GTK_STOCK_SAVE_AS,
                    GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image(GTK_BUTTON(wid), img); 
   wid = gtk_dialog_add_button(GTK_DIALOG(base_w),
       "Edit Original", GTK_RESPONSE_OK); 
   img = gtk_image_new_from_stock(GTK_STOCK_OK, GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image(GTK_BUTTON(wid), img); 
   wid = gtk_dialog_add_button(GTK_DIALOG(base_w),
       GTK_STOCK_CANCEL, GTK_RESPONSE_NONE); 
}

int PrismBackupDialog::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismBackupDialog::doit()
{
   generate_form();
   int rv = 1, sts = gtk_dialog_run(GTK_DIALOG(base_w));
   string msg; 

   if (sts == GTK_RESPONSE_APPLY)
   {
      // save file before proceeding
      GtkWidget* dialog = gtk_file_chooser_dialog_new ("Save File As",
                                         GTK_WINDOW(NULL),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         NULL);
      // add basic filter support
      GtkFileFilter *filt = gtk_file_filter_new ();
      gtk_file_filter_add_pattern (filt, "*");
      gtk_file_filter_set_name(filt,"All Files");
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
      filt = gtk_file_filter_new ();
      gtk_file_filter_add_pattern (filt, "*.fits");
      gtk_file_filter_set_name(filt,"Fits Files");
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
      filt = gtk_file_filter_new ();
      gtk_file_filter_add_pattern (filt, "*.txt");
      gtk_file_filter_add_pattern (filt, "*.dat");
      gtk_file_filter_add_pattern (filt, "*.ascii");
      gtk_file_filter_set_name(filt,"Ascii/Text Files");
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

      if (_filename.length())
      {
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), 
            _filename.c_str());
      }

      if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
      {
         // statically allocated memory- don't free
         gchar * new_fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
	 // honor umask for new copy of file. Since _file_copy uses chmod to set
	 // permissions, subtract the retrieved umask from the 777.
	 mode_t md;
	 umask(md = umask(0));  
         if (new_fname)  rv = _file_copy(_filename, new_fname, 0777 - (int)md);
         if (rv == -3)
	 {
	    msg = "Unable to back up ";
	    msg.append(_filename);
	    msg.append(" as ");
	    msg.append(new_fname);
	    msg.append("- The file already exists and is Read-Only.");
	    theApp->status_message(msg, 1); 
         }
         else if (rv == -2)
	 {
	    msg = "Unable to back up ";
	    msg.append(_filename); 
	    msg.append(" as itself.");
	    theApp->status_message(msg, 1); 
	 }
         else if (rv == -1)
	 {
	    msg = "Unable to back up ";
	    msg.append(_filename); 
	    msg.append("- no filename provided.");
	    theApp->status_message(msg, 1); 
	 }
         else if (rv == 0)
	 {
	    msg = "Copied ";
	    msg.append(_filename);
	    msg.append(" to "); 
	    msg.append(new_fname);
	    msg.append(" and reopening it for editing.");
	    theApp->status_message(msg); 
	    theApp->load_file(new_fname, 0, 1); 
	 }
      }
      gtk_widget_destroy (dialog);
   }
   else if (sts == GTK_RESPONSE_OK)
   {
      // proceed w/o saving file 
      msg = "Unlocking ";
      msg.append(_filename);
      msg.append(" for editing w/o making a backup copy.");
      theApp->status_message(msg);
      rv = 0;
   }
   else
   {
      msg = "Unlock request for ";
      msg.append(_filename);
      msg.append("cancelled."); 
      rv = 1; 
   }
   gtk_widget_destroy(base_w);
   return rv;
}

