// 
//  Copyright (C) 2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMDATAAREA_HH
#define PRISMDATAAREA_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

#include "PrismBaseObj.hh"

class PrismAppendRowCmd; 
class PrismAppendColumnCmd; 
class PrismDataTable;
class PrismDataToolBar; 
class PrismDataJumpCmd; 
class PrismDataSet; 
class PrismImageCmd; 
class PrismHistogramCmd; 
class PrismPlotCmd; 

using namespace std;

class PrismDataArea: public PrismBaseObj
{
  public:
    PrismDataArea(string filename="");
    virtual ~PrismDataArea();
    virtual int show(); 

    int set_data(PrismDataSet* dset, long row=1); 

    int first_page();
    int prev_page();
    int goto_page(); 
    int next_page();
    int last_page();
    int format_column(); 
    int get_meta_data();
    int generate_image(int dialog = -1);
    int generate_plot(int dialog = -1);
    int generate_histogram(int dialog = -1);
    int append_column(int dialog = -1);
    int append_row(int dialog = -1);
    int set_selection(int val=0, int ascol=1); 
    string get_selected_cols(); 

    void set_frame_label(string val);

    void disable_editing(); 
    void enable_editing(); 
    void set_toolbar_sensivity(int btn, bool sensitive);

  protected:
    GtkWidget *vbox; 
//    GtkWidget *hbox; 
    PrismDataTable* table; 
    PrismDataToolBar* dataBtns; 
    PrismDataSet*    _dset; 

    PrismAppendColumnCmd* _appendColumn; 
    PrismAppendRowCmd* _appendRow; 
    PrismHistogramCmd* _histoCmd;
    PrismImageCmd*  _imager;
    PrismPlotCmd*   _plotter;
    PrismDataJumpCmd* _jumpCmd; 
};

#endif 
