// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMDATABLOCK_HH
#define PRISMDATABLOCK_HH

#include <stdio.h>
#include <string>
#include <list>
#include "ascdm.h"

#include "PrismColumnInfraP.hh"


using namespace std; 

typedef struct
{
   string   name;
   string   value;
   string   type;
   string   comment;
   string   units; 
   int      pos; 
   bool     edit; 
} HeaderKey_t, *HeaderKey_p;


class PrismDataColumn;

class PrismDataBlock {
  public:
    PrismDataBlock(dmBlock* block, bool filtered=false, int maxsz=100);
    ~PrismDataBlock();
    
    int  get_no() { return _extnum; }
    string get_name() { return _extname; }
    long get_type()   { return _type; } 
    long get_ncols()  { return _ncols; } 
    long get_totcols()  { return _totcols; } 
    long get_dispcols()  { return _dispcols; } 
    long get_nrows()  { return _nrows; } 
    long get_nkeys()  { return _nkeys; }

    bool is_image()   { return _type == dmIMAGE; }
    PrismDataColumn* get_columns()   { return _cols; }
    HeaderKey_p  get_keys()   { return _keys; }
    string* get_data(long colno, long start, long stop, long &colsback); // user delete[]
    string* get_image_data(long start=0, long stop=0); // user delete[]
    dmBlock* get_block()  { return _block; }

    // header options
    int insert_keyword(string keyname, string value, string type, string comment,
                       string units);
    int delete_keyword(string value);
    int change_keyword_value(string keyname, string value);
    int change_keyword_name(string keyname, string value);
    int change_keyword_comment(string keyname, string value);
    int change_keyword_units(string keyname, string value); 
    bool key_exists(string keyname); 
    int get_key_fields(string keyname, string &value, string &comment);

    int load_header(); 
    int get_extnum()  { return _extnum; }

    string get_errmsg() { return _errmsg; }

    // data extenstion options
    int validate_value(int col, long row, string value); 
    int set_value(int col, long row, string value, long arrpos=-1); 
    int append_row(long copyrow=dmENDOFTABLE);
    int append_column(PrismColumnInfra_p colstruct);                                                                                                             
    int get_column_names(string* &colnames);
    void flush_changes();

  protected:
    int load_columns(); 

    dmBlock         *_block;     // The DM block pointer
    dmBlockType      _type;
    PrismDataColumn *_cols;      // data columns  
    string         _extname;     // extension name
    string         _errmsg;      // error message from validate value
    long           _ncols;
    long           _totcols;     // total number of cols + expanded cols 
    long           _dispcols;    // total number of displayed cols 
    long           _nrows;
    int            _extnum;      // block number 
    int            _maxExpandSz; // maximum size for inline expansion of vector/array

    long           _nkeys;       // number of header keywords 
    HeaderKey_p    _keys;        // header keyword array 
    bool           _filtered;    // is this a filtered block?
    
};


#endif 
