// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismDataJumpCmd.hh"
#include "PrismDataTable.hh"
#include "PrismApp.hh"
#include <iostream>
#include <sstream>

using namespace std;

PrismDataJumpCmd::PrismDataJumpCmd():PrismBaseObj()
{
   _toggle = 0;
}

PrismDataJumpCmd::~PrismDataJumpCmd()
{
}

void PrismDataJumpCmd::create_dialog()
{
   base_w = gtk_dialog_new_with_buttons("Jump to Row/Page", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(7,6, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), tab);

   stringstream ss;
   string rowloc, pageloc;

   _pagecurr = _table->get_current_page();
   _rowcurr = 1 + ((_pagecurr-1) * _table->get_pagesize());  
   _pagemin = _rowmin = 1;
   _pagemax = _table->get_npages(); 
   _rowmax = _table->get_nrows();

   ss << "   (" << _rowcurr << '/' << _rowmax << ')';
   ss >> rowloc; 
   ss.clear();
   ss << "   (" << _pagecurr << '/' << _pagemax << ')';
   ss >> pageloc; 

   GtkWidget* locrowlab = gtk_label_new(rowloc.c_str());
   GtkWidget* locpagelab = gtk_label_new(pageloc.c_str());
   GtkWidget* labtog = gtk_label_new("Jump by");
   _rowtog_w = gtk_radio_button_new_with_label(NULL, "Row");
   _pagetog_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_rowtog_w), "Page");
   if (_toggle) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_pagetog_w), true);
      else      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_rowtog_w), true);
   g_signal_connect(_pagetog_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   GtkWidget* labrow = gtk_label_new("Row");
   GtkAdjustment *row_adj = (GtkAdjustment *) 
      gtk_adjustment_new (_rowcurr, _rowmin, _rowmax, 10.0, 5.0, 0.0);
   _rowspin_w   = gtk_spin_button_new(row_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_rowspin_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_rowspin_w),GTK_UPDATE_IF_VALID);
   g_signal_connect(_rowspin_w, "value-changed", G_CALLBACK(rowspin_value_callback),
                    (void*)this);
   _rowscale_w = gtk_hscale_new(row_adj);
   g_signal_connect(_rowscale_w,"value-changed",G_CALLBACK(rowscale_value_callback),
                    (void*)this);
   gtk_scale_set_draw_value(GTK_SCALE(_rowscale_w), 0);

   GtkWidget* labpage = gtk_label_new("Page");
   GtkAdjustment *page_adj = (GtkAdjustment *) 
      gtk_adjustment_new(_pagecurr, _pagemin, _pagemax, 10.0, 5.0, 0.0);
   _pagespin_w   = gtk_spin_button_new(page_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_pagespin_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_pagespin_w), GTK_UPDATE_IF_VALID);
   g_signal_connect(_pagespin_w, "value-changed", G_CALLBACK(pagespin_value_callback),
                    (void*)this);
   _pagescale_w = gtk_hscale_new(page_adj);
   g_signal_connect(_pagescale_w,"value-changed",G_CALLBACK(pagescale_value_callback),
                    (void*)this);
   gtk_scale_set_draw_value(GTK_SCALE(_pagescale_w), 0);

   gtk_table_attach_defaults(GTK_TABLE(tab), labtog, 0,2,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), _rowtog_w, 2,3,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), _pagetog_w, 3,4,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), labrow, 0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _rowspin_w, 2,4,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), locrowlab, 4,6,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _rowscale_w, 1,5,3,4);
   gtk_table_attach_defaults(GTK_TABLE(tab), labpage, 0,2,5,6);
   gtk_table_attach_defaults(GTK_TABLE(tab), _pagespin_w, 2,4,5,6);
   gtk_table_attach_defaults(GTK_TABLE(tab), locpagelab, 4,6,5,6);
   gtk_table_attach_defaults(GTK_TABLE(tab), _pagescale_w, 1,5,6,7);
   set_sensitive(); 
   show(); 
}

int PrismDataJumpCmd::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismDataJumpCmd::doit(PrismDataTable* table)
{
   int rv  = 0; 

   if (!theApp)             return 1;   // app doesn't exist
   if (!(_table = table))   return 2;    // table doesn't exist

   create_dialog();
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
      if ((_toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pagetog_w))))
         rv= _table->load_page((long)gtk_spin_button_get_value(
	    GTK_SPIN_BUTTON(_pagespin_w)));
      else
      {
	 /**
         // find page with requested row
	 long cr = (long)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_rowspin_w)); 
         long ps = _table->get_pagesize();
         if (ps)   (cr / ps) + 1; 
         rv = _table->load_page(cr); 
         **/
	 rv = _table->load_row((long)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_rowspin_w))); 
      }
   }
   gtk_widget_destroy(base_w);
   return rv;
}

int PrismDataJumpCmd::set_pagescale_value(long val)
{
   if ((val < _pagemin) || (val > _pagemax)) return 1; 
   if (val != (long) gtk_range_get_value(GTK_RANGE(_pagescale_w)))
      gtk_range_set_value(GTK_RANGE(_pagescale_w), val); 
   return 0; 
}

int PrismDataJumpCmd::set_pagespin_value(long val)
{
   if ((val < _pagemin) || (val > _pagemax)) return 1; 
   if (val != (long) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_pagespin_w)))
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(_pagespin_w), val);
   return 0; 
}

int PrismDataJumpCmd::set_rowscale_value(long val)
{
   if ((val < _rowmin) || (val > _rowmax)) return 1; 
   if (val != (long) gtk_range_get_value(GTK_RANGE(_rowscale_w)))
      gtk_range_set_value(GTK_RANGE(_rowscale_w), val); 
   return 0; 
}

int PrismDataJumpCmd::set_rowspin_value(long val)
{
   if ((val < _rowmin) || (val > _rowmax)) return 1; 
   if (val != (long) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_rowspin_w)))
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(_rowspin_w), val);
   return 0; 
}

int PrismDataJumpCmd::pagespin_value_callback(GObject *object, gpointer user_data)
{
   GtkSpinButton *ent = GTK_SPIN_BUTTON(object);
   PrismDataJumpCmd* cmd = (PrismDataJumpCmd*)user_data;

   if (!(ent&&cmd))  return 1;
   long val = (long)gtk_spin_button_get_value(ent);
   return cmd->set_pagescale_value(val); 
}

int PrismDataJumpCmd::pagescale_value_callback(GObject *object, gpointer user_data)
{
   GtkScale *ent = GTK_SCALE(object);
   PrismDataJumpCmd* cmd = (PrismDataJumpCmd*)user_data;

   if (!(ent&&cmd))  return 1;
   long val = (long) gtk_range_get_value(GTK_RANGE(ent));
   return cmd->set_pagespin_value(val); 
}


int PrismDataJumpCmd::rowspin_value_callback(GObject *object, gpointer user_data)
{
   GtkSpinButton *ent = GTK_SPIN_BUTTON(object);
   PrismDataJumpCmd* cmd = (PrismDataJumpCmd*)user_data;

   if (!(ent&&cmd))  return 1;
   long val = (long)gtk_spin_button_get_value(ent);
   return cmd->set_rowscale_value(val); 
}

int PrismDataJumpCmd::rowscale_value_callback(GObject *object, gpointer user_data)
{
   GtkScale *ent = GTK_SCALE(object);
   PrismDataJumpCmd* cmd = (PrismDataJumpCmd*)user_data;

   if (!(ent&&cmd))  return 1;
   long val = (long) gtk_range_get_value(GTK_RANGE(ent));
   return cmd->set_rowspin_value(val); 
}

int PrismDataJumpCmd::toggled_value_callback(GObject *object, gpointer user_data)
{
   PrismDataJumpCmd* cmd = (PrismDataJumpCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->set_sensitive(); 
   return 0;
}

void PrismDataJumpCmd::set_sensitive()
{
   _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_pagetog_w));

   gtk_widget_set_sensitive(GTK_WIDGET(_rowscale_w), !_toggle); 
   gtk_widget_set_sensitive(GTK_WIDGET(_rowspin_w), !_toggle); 
   gtk_widget_set_sensitive(GTK_WIDGET(_pagescale_w), _toggle); 
   gtk_widget_set_sensitive(GTK_WIDGET(_pagespin_w), _toggle); 
}


