// 
//  Copyright (C) 2007-2008,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMDATASET_HH
#define PRISMDATASET_HH

#include <stdio.h>
#include <string>
#include <list>
#include "ascdm.h"

using namespace std; 

class PrismDataBlock;

typedef struct
{
   string name;
   string type;
   string dim;
} ExtensionInfo_t, *ExtensionInfo_p; 


class PrismDataSet {
  public:
    PrismDataSet(string filename="");
    ~PrismDataSet();

    int open_extension(int posn);
    int open_extension(string ename);
    int browse_extensions(); 
    int get_extensions(list<ExtensionInfo_t> &ext_list);  

    string get_filename()     { return _fname;     } 
    string get_current_extname();
    int  get_nblocks()        { return _nblocks;   }
    int  get_filter_block()   { return _filtblock; }
    int  get_current_block()  { return _currBlock; }
    int  get_status()         { return _status; }
    PrismDataBlock* get_block()  { return _block; }

    long get_nrows();
    int  get_ncols();
    int  get_editmode()       { return _editmode; }
    int  set_editmode(); 
    void set_modified(int val=1);
    int  get_modified()       { return _modified; } 

    bool is_ascii()           { return _ascii; }

  protected:
    int open_dataset(); 
    void flush_buffer();

    string _fname;
    string _filter;
    string _currExtname; 
    dmDataset *_dataset;
    PrismDataBlock *_block; 
    dmBlock *_fblock; 
    ExtensionInfo_p _extensions; 
    
    int    _filtblock;    // filtered block number
    int    _nblocks;
    int    _currBlock;
    int    _status; 
    int    _editmode;     // 0 = can't, 1 = enabled, 2 = edited 
    bool   _modified;     // true if file was modified 
    bool   _firstunlock;  // query user at first unlock 
    bool   _ascii;        // ascii or text file (no header edits)
   
};


#endif 
