// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismApp.hh"
#include "PrismEditMenu.hh"

using namespace std;

PrismEditMenu::PrismEditMenu(): PrismBaseObj()
{
   GtkWidget *sep = NULL; 

   base_w = gtk_menu_item_new_with_mnemonic("_Edit");

   menu = gtk_menu_new();

   menuUnlock = gtk_menu_item_new_with_mnemonic("_Enable File Editing");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuUnlock, 0);
   g_signal_connect_swapped (G_OBJECT (menuUnlock), "activate",
                             G_CALLBACK (unlock_cb),
                             (gpointer) NULL);
   menuLock = gtk_menu_item_new_with_mnemonic("_Disable File Editing");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuLock, 1);
   g_signal_connect_swapped (G_OBJECT (menuLock), "activate",
                             G_CALLBACK (lock_cb),
                             (gpointer) NULL);

   sep = gtk_separator_menu_item_new(); 
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, 2);

   menuPrefs = gtk_menu_item_new_with_mnemonic("_Preferences");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuPrefs, 3);

   GtkTooltips* tips = theApp->get_tips_group();
   if (tips)
   {
      gtk_tooltips_set_tip(tips, menuLock, 
         "locks the file in the current tab to prevent editing",NULL);
      gtk_tooltips_set_tip(tips, menuUnlock, 
         "unlocks the file in the current tab to allow editing",NULL);
      gtk_tooltips_set_tip(tips, menuPrefs, 
         "edit/view configurable preferences",NULL);
   }

   g_signal_connect_swapped (G_OBJECT (menuPrefs), "activate",
                             G_CALLBACK (preferences_cb),
                             (gpointer) NULL);

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

PrismEditMenu::~PrismEditMenu()
{
   
}

int PrismEditMenu::show()
{
   gtk_widget_show(menuPrefs);
   gtk_widget_show(menuLock);
   gtk_widget_show(menuUnlock);
   gtk_widget_show(base_w); 
   return 0; 
}

int PrismEditMenu::preferences_cb()
{
   theApp->launch_preferences_dialog(); 
   return 0; 
}

int PrismEditMenu::lock_cb()
{
   theApp->lock_current_tab(0);
   return 0;
}

int PrismEditMenu::unlock_cb()
{
   theApp->lock_current_tab(1);
   return 0;
}


