// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHeaderTable.hh"
#include "PrismDataBlock.hh"
#include "PrismApp.hh"

using namespace std;

PrismHeaderTable::PrismHeaderTable(string extname):PrismBaseObj()
{
   articles = NULL;
   model = gtk_list_store_new(NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING,
                              G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
			      G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
   treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(model)));
   table = GTK_WIDGET(treeview);
   g_object_unref (model);

   gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview) ,1);
   GtkTreeViewColumn *column;

   rndr_name = gtk_cell_renderer_text_new();
   g_signal_connect(rndr_name, "edited",
                    G_CALLBACK(PrismHeaderTable::cell_edited), this);
   g_object_set_data(G_OBJECT(rndr_name), "column", (gint *)KEY_NAME);
   column = gtk_tree_view_column_new_with_attributes ("Name",
      rndr_name, "text", KEY_NAME, "editable", KEY_UNLOCKED, NULL); 
   gtk_tree_view_column_set_sort_column_id (column, KEY_NAME);
   gtk_tree_view_append_column(treeview, column);
   gtk_tree_view_column_set_resizable(column, true); 
   gtk_tree_view_column_set_reorderable(column, true); 

   rndr_val = gtk_cell_renderer_text_new();
   g_signal_connect(rndr_val, "edited",
                    G_CALLBACK(PrismHeaderTable::cell_edited), this);
   g_object_set_data(G_OBJECT(rndr_val), "column", (gint *)KEY_VALUE);
   column = gtk_tree_view_column_new_with_attributes ("Value",
      rndr_val, "text", KEY_VALUE, "editable", KEY_UNLOCKED, NULL);
   gtk_tree_view_append_column(treeview, column);
   gtk_tree_view_column_set_resizable(column, true); 
   gtk_tree_view_column_set_reorderable(column, true); 

   rndr_type = gtk_cell_renderer_text_new();
   g_signal_connect(rndr_type, "edited",
                    G_CALLBACK(PrismHeaderTable::cell_edited), this);
   g_object_set_data(G_OBJECT(rndr_type), "column", (gint *)KEY_TYPE);
   column = gtk_tree_view_column_new_with_attributes ("Type",
      rndr_type, "text", KEY_TYPE, "editable", KEY_UNLOCKED, NULL); 
   gtk_tree_view_append_column(treeview, column);
   gtk_tree_view_column_set_resizable(column, true); 
   gtk_tree_view_column_set_reorderable(column, true); 

   rndr_cmt = gtk_cell_renderer_text_new();
   g_signal_connect(rndr_cmt, "edited",
                    G_CALLBACK(PrismHeaderTable::cell_edited), this);
   g_object_set_data(G_OBJECT(rndr_cmt), "column", (gint *)KEY_COMMENT);
   column = gtk_tree_view_column_new_with_attributes ("Comment",
      rndr_cmt, "text", KEY_COMMENT, "editable", KEY_UNLOCKED, NULL); 
   gtk_tree_view_append_column(treeview, column);
   gtk_tree_view_column_set_resizable(column, true); 
   gtk_tree_view_column_set_reorderable(column, true); 

   rndr_units = gtk_cell_renderer_text_new();
   g_signal_connect(rndr_units, "edited",
                    G_CALLBACK(PrismHeaderTable::cell_edited), this);
   g_object_set_data(G_OBJECT(rndr_units), "column", (gint *)KEY_UNITS);
   column = gtk_tree_view_column_new_with_attributes ("Units",
      rndr_units, "text", KEY_UNITS, "editable", KEY_UNLOCKED, NULL); 
   gtk_tree_view_append_column(treeview, column);
   gtk_tree_view_column_set_resizable(column, true); 
   gtk_tree_view_column_set_reorderable(column, true); 

   rndr_key = gtk_cell_renderer_text_new();
   g_signal_connect(rndr_key, "edited",
                    G_CALLBACK(PrismHeaderTable::cell_edited), this);
   g_object_set_data(G_OBJECT(rndr_key), "column", (gint *)KEY_POS);
   column = gtk_tree_view_column_new_with_attributes ("Key NO",
      rndr_key, "text", KEY_POS, "editable", KEY_UNLOCKED, NULL); 
   gtk_tree_view_column_set_sort_column_id(column, KEY_POS);
   gtk_tree_view_append_column(treeview, column);
   gtk_tree_view_column_set_resizable(column, true); 
   gtk_tree_view_column_set_reorderable(column, true); 

   // load settings for displayed columns
   set_column_visibility(); 

   base_w = gtk_scrolled_window_new (NULL,NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(base_w),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(base_w), table);
   _origkeys = _nkeys = 0;
}

PrismHeaderTable::~PrismHeaderTable()
{
   if (articles)
   {
      HeaderTableItem* hdr_p = NULL; 
      for (int jj = articles->len; jj--; )
         if ((hdr_p = &g_array_index(articles,HeaderTableItem,jj)) != NULL)
         {
            g_free(hdr_p->name);
            g_free(hdr_p->type);
            g_free(hdr_p->value);
            g_free(hdr_p->comment);
            g_free(hdr_p->units);
         }
      g_array_free(articles, TRUE);
      articles = NULL; 
   }
}

int PrismHeaderTable::load_entries(PrismDataBlock* blk, string setid)
{
   HeaderTableItem sample;
   HeaderKey_p keys;

   if (!blk) return 1;  // no block
   _block = blk; 

   _origkeys = _nkeys = _block->get_nkeys();
   keys = _block->get_keys(); 
   
   // remove old entries
   gtk_list_store_clear(model);
   gtk_tree_view_columns_autosize(treeview);
   if (articles)
   {
      HeaderTableItem* hdr_p = NULL; 
      for (int jj = articles->len; jj--; )
         if ((hdr_p = &g_array_index(articles,HeaderTableItem,jj)) != NULL)
         {
            g_free(hdr_p->name);
            g_free(hdr_p->type);
            g_free(hdr_p->value);
            g_free(hdr_p->comment);
            g_free(hdr_p->units);
         }
      g_array_free(articles, TRUE);
      articles = NULL; 
   }

   // no header keys to display
   if (_nkeys == 0)  return 0; 

   articles = g_array_sized_new (FALSE, FALSE, sizeof (HeaderTableItem), _nkeys);
   string tmp; 
   int setpos;
   bool hide_raw; 
   if (theApp->get_preference("hdr.filter.raw", hide_raw))
      hide_raw = false;
   for (int rr =0; rr < _nkeys; rr++)
   {
      // display raw keys unless disabled- always display real keys & comments
      if (keys[rr].edit || keys[rr].type == "comment" || !hide_raw)
      {
         sample.name = g_strdup(keys[rr].name.c_str());
         sample.value = g_strdup(keys[rr].value.c_str());
         sample.type = g_strdup(keys[rr].type.c_str());
         sample.comment = g_strdup(keys[rr].comment.c_str());
         sample.units = g_strdup(keys[rr].units.c_str());
         sample.unlocked = sample.editable = keys[rr].edit;
         sample.pos = keys[rr].pos;
         g_array_append_vals (articles, &sample, 1);
         if (setid == sample.name)   setpos = rr; 
      }
   }
   if (!setid.length()) setpos = -1;
   GtkTreePath * set_path = NULL;
   
   for (int i = 0; i < articles->len; i++)
   {
      gtk_list_store_append(model, &iter);
      gtk_list_store_set(model, &iter,
                         KEY_NAME,
                         g_array_index (articles, HeaderTableItem, i).name,
                         KEY_VALUE,
                         g_array_index (articles, HeaderTableItem, i).value,
                         KEY_TYPE,
                         g_array_index (articles, HeaderTableItem, i).type,
                         KEY_COMMENT,
                         g_array_index (articles, HeaderTableItem, i).comment,
                         KEY_UNITS,
                         g_array_index (articles, HeaderTableItem, i).units,
                         KEY_POS,
                         g_array_index (articles, HeaderTableItem, i).pos,
                         KEY_EDITABLE,
                         g_array_index (articles, HeaderTableItem, i).editable,
                         KEY_UNLOCKED,
                         g_array_index (articles, HeaderTableItem, i).unlocked,
                         -1);
       if (i == setpos)  // get path of keyword to highlight 
          set_path = gtk_tree_model_get_path((GtkTreeModel*)model, &iter);
   }
   if (setpos != -1 && set_path)     // hightlight requested keyword 
      gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview), set_path, NULL, FALSE);

   set_column_visibility(); 
}

int PrismHeaderTable::show()
{
   gtk_widget_show_all(GTK_WIDGET(treeview));
   return 0;
}


void PrismHeaderTable::cell_edited(GtkCellRendererText *cell, 
   const gchar *path_string, const gchar *new_text, gpointer data)
{
   PrismHeaderTable* tab = (PrismHeaderTable *)data;
   GtkTreeModel *mod = (GtkTreeModel *) tab->get_model();
   GtkTreePath *path = gtk_tree_path_new_from_string (path_string);
   GtkTreeIter iter;

   GArray      *art = tab->get_articles(); 
   gint column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "column"));
   gtk_tree_model_get_iter (mod, &iter, path);
   gchar *keyname; 
   gtk_tree_model_get(mod, &iter, 0, &keyname, -1);
   gint ii, rval; 
   switch (column)
   {
      case KEY_NAME:
      {
	 gint ndx; 
	 gtk_tree_model_get(mod, &iter, KEY_POS, &ndx, -1);
	 if (ndx > tab->get_default_num_keys())
	 {
	    // this was a column we added, set the override to allow the
	    // user to create a new column here, if the old keyname is blank
            gchar *typeval, *descval, *valueval, *unitval; 
            gtk_tree_model_get(mod, &iter, KEY_TYPE, &typeval, -1);
            gtk_tree_model_get(mod, &iter, KEY_VALUE, &valueval, -1);
            gtk_tree_model_get(mod, &iter, KEY_COMMENT, &descval, -1);
            gtk_tree_model_get(mod, &iter, KEY_UNITS, &unitval, -1);
	    if ((rval = tab->add_keyword(new_text,valueval,typeval,descval,unitval)) == 0)
	    { 
	       // key inserted- update table
               tab->refresh(new_text); 
	       theApp->set_modified(); 
	    }
            g_free(valueval);
            g_free(typeval);
            g_free(descval);
            g_free(unitval);
	 }
         else if ((rval = tab->modify_keyword(keyname, new_text, column)) == 0)
         {
            gchar *old_text;
            gtk_tree_model_get(mod, &iter, column, &old_text, -1);
            g_free (old_text);
	    ii = gtk_tree_path_get_indices (path)[0];
            g_free (g_array_index (art, HeaderTableItem, ii).name);
            g_array_index (art, HeaderTableItem, ii).name = g_strdup(new_text);
            gtk_list_store_set(GTK_LIST_STORE(mod), &iter, column,
               g_array_index(art, HeaderTableItem, ii).name, -1);
         }
	 else if (rval == 4) 
	 {
	    string sts = "Keyword "; 
	    sts.append(new_text);
	    sts.append(" already exists as a keyword in the dataset");
            theApp->status_message(sts,1);
         }
      }
      break; 
      case KEY_VALUE:
      {
         if (tab->modify_keyword(keyname, new_text, column) == 0)
	 {
            gchar *old_text;
            gtk_tree_model_get(mod, &iter, column, &old_text, -1);
            g_free (old_text);
	    ii = gtk_tree_path_get_indices (path)[0];
            g_free (g_array_index (art, HeaderTableItem, ii).value);
            g_array_index (art, HeaderTableItem, ii).value = g_strdup(new_text);
            gtk_list_store_set(GTK_LIST_STORE(mod), &iter, column,
                g_array_index(art, HeaderTableItem, ii).value, -1);
	    theApp->set_modified(); 
         }
	 else
	 {
	    // conversion failure 
	    string sts = "Unable to convert value [";
	    sts.append(new_text);
	    sts.append("] to the specified type for key ");
	    sts.append(keyname);
	    theApp->status_message(sts,1); 
	 }
      }
      break; 
      case KEY_UNITS:
      {
         if (tab->modify_keyword(keyname, new_text, column) == 0)
	 {
            gchar *old_text;
            gtk_tree_model_get(mod, &iter, column, &old_text, -1);
            g_free (old_text);
	    ii = gtk_tree_path_get_indices (path)[0];
            g_free (g_array_index (art, HeaderTableItem, ii).units);
            g_array_index (art, HeaderTableItem, ii).units = g_strdup(new_text);
            gtk_list_store_set(GTK_LIST_STORE(mod), &iter, column,
                g_array_index(art, HeaderTableItem, ii).units, -1);
	    theApp->set_modified(); 
         }
	 else
	 {
	    string sts = "Unable to set the units field to the value [";
	    sts.append(new_text);
	    sts.append("] for key ");
	    sts.append(keyname);
	    theApp->status_message(sts,1); 
	 }
      }
      break; 
      break; 
      case KEY_COMMENT:
      {
         if (tab->modify_keyword(keyname, new_text, column) == 0)
         {
            gchar *old_text;
            gtk_tree_model_get(mod, &iter, column, &old_text, -1);
            g_free (old_text);
	    ii = gtk_tree_path_get_indices (path)[0];
            g_free (g_array_index (art, HeaderTableItem, ii).comment);
            g_array_index (art, HeaderTableItem, ii).comment = g_strdup(new_text);
            gtk_list_store_set(GTK_LIST_STORE(mod), &iter, column,
                g_array_index(art, HeaderTableItem, ii).comment, -1);
	    theApp->set_modified(); 
         }
	 else
	 {
	    // failed to change comment
	    string sts = "Unable to set the comment field to the value [";
	    sts.append(new_text);
	    sts.append("] for key ");
	    sts.append(keyname);
	    theApp->status_message(sts,1); 
	 }
      }
      break; 
      default:
      break; 
   }
   gtk_tree_path_free (path);
   g_free(keyname);
}

int PrismHeaderTable::add_item(string keyname, string value, string type, 
   string comment, string units)
{
   if (keyname.length())
   {
      int rv; 
      // if we actually add a keyword- refresh the display 
      if (!(rv = add_keyword(keyname, value, type, comment, units)))
      {
	 refresh(keyname); 
	 theApp->set_modified(); 
      }
      return rv; 
   }

   // add a place holder in the table, the entry won't get added til the
   // keyword is set 
   HeaderTableItem sample;
   sample.name = g_strdup(keyname.c_str());
   sample.value = g_strdup(value.c_str());
   sample.type = g_strdup(type.c_str());
   sample.comment = g_strdup(comment.c_str());
   sample.units = g_strdup(units.c_str());
   sample.pos = ++_nkeys; 

   g_array_append_vals (articles, &sample, 1);    // store new entry
   gtk_list_store_append (model, &iter);          // add to display
   gtk_list_store_set (model, &iter,
                       KEY_NAME, sample.name,
                       KEY_VALUE, sample.value,
                       KEY_TYPE, sample.type,
                       KEY_COMMENT, sample.comment,
		       KEY_UNITS, sample.units,
                       KEY_POS, sample.pos,
                       KEY_EDITABLE, 1,
                       -1);

   // position cursor on keyword cell
   GtkTreePath * path = gtk_tree_model_get_path(GTK_TREE_MODEL(model), &iter);
   GtkTreeViewColumn *col = gtk_tree_view_get_column(GTK_TREE_VIEW (treeview), 0);
   GtkCellRenderer *focus_cell = NULL;
   gtk_tree_view_set_cursor(GTK_TREE_VIEW (treeview), path,
                                    col, TRUE);
   gtk_widget_grab_focus(GTK_WIDGET(treeview));
   return 0;
}


int PrismHeaderTable::modify_keyword(string keyname, string value, int field)
{
   if (!_block)  return 1;   // no block - failed
   int rv = 2;               // default to unsupported field
   switch(field)
   {
      case KEY_NAME:
         rv = _block->change_keyword_name(keyname, value);
      break;
      case KEY_VALUE: 
         rv =_block->change_keyword_value(keyname, value);
      break;
      case KEY_COMMENT:
         rv = _block->change_keyword_comment(keyname, value);
      break;
      case KEY_UNITS:
         rv = _block->change_keyword_units(keyname, value);
      break;
      default:
         // do nothing
      break;
   }
   if (rv == 0)  theApp->set_modified();
   return rv;  
}

int PrismHeaderTable::add_keyword(string keyname, string value, string type,
   string comment, string units)
{
   if (!_block)  return 1;   // no block - failed
   int rv = _block->insert_keyword(keyname, value, type, comment, units); 
   switch(rv)
   {
      case 0:
         theApp->status_message("Added keyword " + keyname + " to extension " + 
                                _block->get_name());
      break; 
      case 3: 
         theApp->status_message("Error: Unable to add duplicate keyword " + keyname +
                                 " to extension " + _block->get_name(),1);
      break; 
      case 4:
         theApp->status_message("Error: The specified data type [" + type + "]  is currently unsupported- keyword not added",1); 
      break; 
      case 6:
         theApp->status_message("Error: Invalid entry [" + value + "] for long data type- keyword not added",1); 
      break; 
      case 7:
         theApp->status_message("Error: Invalid entry [" + value + "] for double data type- keyword not added",1); 
      break; 
      case 8:
         theApp->status_message("Error: Invalid entry [" + value + "] for float data type- keyword not added",1); 
      break; 
      case 9:
         theApp->status_message("Error: Invalid entry [" + value + "] for short data type- keyword not added",1); 
      break; 
      case 10:
         theApp->status_message("Error: Invalid entry [" + value + "] for boolean data type- keyword not added",1); 
      break; 
      default:
      break; 
   }
   theApp->set_modified(); 
   return rv; 
}

int PrismHeaderTable::delete_keyword(string keyname)
{
   if (!_block)  return 1;   // no block - failed
   int rv = _block->delete_keyword(keyname);

   // regenerate display to remove the entry from the table 
   if (!rv)
   {
      theApp->status_message("Deleted key " + keyname + " from extension " + _block->get_name());
      refresh();
      theApp->set_modified(); 
   }
   else
      theApp->status_message("Unable to delete key " + keyname + " from extension " + _block->get_name(),1);
   return rv;    
}

int PrismHeaderTable::set_column_visibility()
{
   if (!theApp)   return 1;    // no values provided;

   GtkTreeViewColumn *col;
   bool  vals[NUM_COLUMNS];

   if (theApp->get_preference("hdr.display.keyname", vals[KEY_NAME]))
      vals[KEY_NAME] = true;
   if (theApp->get_preference("hdr.display.value", vals[KEY_VALUE]))
      vals[KEY_VALUE] = true;
   if (theApp->get_preference("hdr.display.type", vals[KEY_TYPE]))
      vals[KEY_TYPE] = true;
   if (theApp->get_preference("hdr.display.comment", vals[KEY_COMMENT]))
      vals[KEY_COMMENT] = true;
   if (theApp->get_preference("hdr.display.units", vals[KEY_UNITS]))
      vals[KEY_UNITS] = true;
   if (theApp->get_preference("hdr.display.keyno", vals[KEY_POS]))
      vals[KEY_POS] = true;
   
   if ((col = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), KEY_NAME)) != NULL)
      gtk_tree_view_column_set_visible(col, vals[KEY_NAME]);
   if ((col = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), KEY_VALUE)) != NULL)
      gtk_tree_view_column_set_visible(col, vals[KEY_VALUE]);
   if ((col = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), KEY_TYPE)) != NULL)
      gtk_tree_view_column_set_visible(col, vals[KEY_TYPE]);
   if ((col = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview),KEY_COMMENT))!= NULL)
      gtk_tree_view_column_set_visible(col, vals[KEY_COMMENT]);
   if ((col = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), KEY_UNITS)) != NULL)
      gtk_tree_view_column_set_visible(col, vals[KEY_UNITS]);
   if ((col = gtk_tree_view_get_column(GTK_TREE_VIEW(treeview), KEY_POS)) != NULL)
      gtk_tree_view_column_set_visible(col, vals[KEY_POS]);
   return 0; 
}


bool PrismHeaderTable::key_exists(string keyname)
{
   if (!_block)  return false;
   if (!keyname.length())  return false; 

   return (_block->key_exists(keyname)); 
}

int PrismHeaderTable::get_key_fields(string keyname, string &value, 
   string &comment)
{
   if (!_block)  return 1;
   if (!keyname.length())  return 2;
   return (_block->get_key_fields(keyname, value, comment));
}

int PrismHeaderTable::get_selected_key(string &keyname)
{
   if (!_block)  return 1;   // block doesn't exist 
   GtkTreeSelection* gts = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   GtkTreeIter it1;
   GtkTreeModel *mdl = GTK_TREE_MODEL(model);

   if (!gts)  return 2;      // no tree selection 

   if (gtk_tree_selection_get_selected(gts, &mdl, &it1))
   {
      gchar *name = NULL; 
      gtk_tree_model_get(GTK_TREE_MODEL(model), &it1, KEY_NAME, &name, -1);
      if (name)
      {
         keyname = name; 
         g_free(name);
      }
      return 0; 
   }
   return 1; 
}


void PrismHeaderTable::refresh(string setid)
{
   if (_block)
   {
      _block->load_header(); 
      load_entries(_block, setid);
   }
}

void PrismHeaderTable::disable_editing()
{
   gtk_tree_model_get_iter_first (GTK_TREE_MODEL(model), &iter);
   if (articles)
      for (int i = 0; i < articles->len; i++)
      { 
         gtk_list_store_set(model, &iter, KEY_UNLOCKED, false, -1);
         gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
      }
}

void PrismHeaderTable::enable_editing()
{
   gtk_tree_model_get_iter_first (GTK_TREE_MODEL(model), &iter);
   gboolean ed; 
   if (articles)
      for (int i = 0; i < articles->len; i++)
      { 
         // copy value from editable to unlocked 
         gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, KEY_EDITABLE, &ed, -1);
         gtk_list_store_set(model, &iter, KEY_UNLOCKED, ed, -1);
         gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
      }
}



