// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHistogramCmd.hh"
#include "PrismDataSet.hh"
#include "PrismDataBlock.hh"
#include "PrismDataColumn.hh"

#include "PrismApp.hh"

#include <gtkextra/gtksheet.h>
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <numpy/noprefix.h>
#include <fstream>

using namespace std;


PrismHistogramCmd::PrismHistogramCmd():PrismBaseObj()
{
   _chipsId = ""; 
   _behavior = 2; 
   _lstyle = 1; 
   _dvis = 1; 
   _sstyle = 0; 
   _fstyle = 0;
   _lthick = 1.0; 
   _sthick = 5; 
   _lcolord = _scolord = _fcolord = true; 
   _shell_py = false; 
   _lcolor = _scolor = "white";
   _fopacity = .5; 
   _bins = 50;
   _autobin = true; 
}

PrismHistogramCmd::~PrismHistogramCmd()
{
}

void PrismHistogramCmd::generate_form()
{
   base_w = gtk_dialog_new_with_buttons ("Histogram Dialog", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
      GTK_STOCK_APPLY, GTK_RESPONSE_APPLY, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

   _vbox_w = gtk_vbox_new (FALSE,5); 
   gtk_widget_set_size_request(base_w, 650, 440);
   _scrolled_w = gtk_scrolled_window_new (NULL,NULL);
   gtk_widget_set_size_request(_vbox_w, 650, -1); 

   _frm_w = gtk_frame_new(" Histogram Settings "); 
   gtk_box_pack_start(GTK_BOX(_vbox_w), _frm_w, FALSE, FALSE, 0);
   gtk_widget_show(_frm_w);
   gtk_widget_show(_vbox_w);
   GtkWidget* table = gtk_table_new(11,20, true); 
   gtk_container_add(GTK_CONTAINER(_frm_w), table);

   GtkWidget* label_w = gtk_label_new("Chips Server Id");
   GtkWidget* cb_w = gtk_combo_box_entry_new_text();
   _launch = gtk_check_button_new_with_label("Launch");
   GtkWidget* lab2_w = gtk_label_new("Interactive Shell");

   // status message area 
   _status_w = gtk_label_new("");
   gtk_box_pack_start(GTK_BOX(_vbox_w), _status_w, FALSE, FALSE, 0);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled_w);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled_w),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(_scrolled_w), _vbox_w);
   gtk_widget_show(_status_w);

   // load combo box w/ possible access points
   list<string>::iterator ii = _plotters.begin();
   for(; ii !=_plotters.end(); ++ii)
      gtk_combo_box_append_text(GTK_COMBO_BOX(cb_w), (*ii).c_str());

   // set default xpa access name
   _entry = GTK_ENTRY(GTK_BIN(cb_w)->child);
   if (_entry)  gtk_entry_set_text(_entry, _chipsId.c_str()); 

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_launch), _instantiate); 

   _shell_no_w = gtk_radio_button_new_with_label(NULL, "None");
   _shell_py_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_shell_no_w),"Python");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_shell_no_w), true);

   GtkWidget* cvxl_w = gtk_label_new("Column");
   _cvx_w = gtk_combo_box_entry_new_text();
   set_default_column_name(true);
   GtkWidget* cvyl_w = gtk_label_new("Bins");
   _autobin_w = gtk_check_button_new_with_label("Autobin");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_autobin_w), _autobin); 
   g_signal_connect(_autobin_w,"toggled",G_CALLBACK(autobin_toggle_callback),
                    (void*)this);
   GtkAdjustment *th_adj = (GtkAdjustment *)
      gtk_adjustment_new(_bins, 2, 1000.0, 1.0, 10.0, 0.0);
   _cvy_w = gtk_spin_button_new(th_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_cvy_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_cvy_w),
      GTK_UPDATE_IF_VALID);
   set_bin_button_sensitive();

   GtkWidget* ll_w = gtk_label_new("Line Style");
   _lstyle_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 0, "noline");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 1, "solid");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 2, "dot");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 3, "dotlongdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 4, "dotshortdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 5, "longdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 6, "shortdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 7, "shortdashlongdash");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_lstyle_w), _lstyle);
   GtkWidget* lth_w = gtk_label_new("Thickness");
   th_adj = (GtkAdjustment *)
      gtk_adjustment_new(_lthick, 0.5, 10.0, 1.0, 1.0, 0.0);
   _lthick_w   = gtk_spin_button_new(th_adj, 1.0, 1);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_lthick_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_lthick_w),
      GTK_UPDATE_IF_VALID);
   GdkColor clr; 
   GtkWidget* lc_w = gtk_label_new("Color");
   gdk_color_parse(_lcolor.c_str(), &clr);
   _lcolor_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_lcolor_w), 
      "Line Color Selection"); 
   _lcolord_w = gtk_check_button_new_with_label("Default");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_lcolord_w), _lcolord); 
   g_signal_connect(_lcolord_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   GtkWidget* sl_w = gtk_label_new("Symbol Style");
   _sstyle_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 0, "none");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 1, "cross");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 2, "diamond");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 3, "downtriangle");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 4, "circle");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 5, "plus");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 6, "square");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 7, "uptriangle");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 8, "point");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 9, "arrow");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_sstyle_w), _sstyle);
   GtkWidget* sth_w = gtk_label_new("Size");
   th_adj = (GtkAdjustment *)gtk_adjustment_new(_sthick,1.0,100.0,1.0,5.0,0.0);
   _sthick_w   = gtk_spin_button_new(th_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_sthick_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_sthick_w),
      GTK_UPDATE_IF_VALID);
   GtkWidget* sc_w = gtk_label_new("Color");
   gdk_color_parse(_scolor.c_str(), &clr);
   _scolor_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_scolor_w), 
      "Symbol Color Selection"); 
   _scolord_w = gtk_check_button_new_with_label("Default");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scolord_w), _scolord); 
   g_signal_connect(_scolord_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   _dvis_w = gtk_check_button_new_with_label("Droplines");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_dvis_w), _dvis); 
   GtkWidget* fl_w = gtk_label_new("Fill Style");
   _fstyle_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 0, "no fill");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 1, "solid");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 2, "updiagonal");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 3, "downdiagonal");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 4, "horizontal");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 5, "vertical");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 6, "crisscross");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 7, "brick");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 8, "grid");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 9, "hexagon");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 10, "polkadot");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 11, "zigzag");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 12, "wave");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 13, "flower");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 14, "userfill1");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 15, "userfill2");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_fstyle_w), 16, "userfill3");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_fstyle_w), _fstyle);

   GtkWidget* fo_w = gtk_label_new("Opacity");
   th_adj = (GtkAdjustment *)gtk_adjustment_new(_fopacity,0,1,.05,.1,0);
   _fopacity_w   = gtk_spin_button_new(th_adj, 1.0, 3);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_fopacity_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_fopacity_w),
      GTK_UPDATE_IF_VALID);
   GtkWidget* fc_w = gtk_label_new("Fill Color");
   gdk_color_parse(_fcolor.c_str(), &clr);
   _fcolor_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_fcolor_w), 
      "Histogram Fill Color Selection"); 
   _fcolord_w = gtk_check_button_new_with_label("Default");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_fcolord_w), _fcolord); 
   g_signal_connect(_fcolord_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   // behavior toggles 
   GtkWidget* bh_w = gtk_label_new("Behavior");
   _oplot_w = gtk_radio_button_new_with_label(NULL, "Overplot");
   _nframe_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_oplot_w),"New Window");
   _erase_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_oplot_w),"Erase Existing"); 
   if (_behavior == 0) 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_oplot_w), true);
   else if (_behavior == 1) 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_nframe_w), true);
   else
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_erase_w), true);

   set_color_buttons_sensitive(); 

   gtk_table_attach_defaults(GTK_TABLE(table), cvxl_w, 1,4,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _cvx_w, 4,8,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), cvyl_w, 10,11,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _autobin_w, 12,15,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _cvy_w, 15,18,1,2);

   gtk_table_attach_defaults(GTK_TABLE(table), ll_w, 1,4,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lstyle_w, 4,8,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), lth_w, 8,10,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lthick_w, 10,12,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), lc_w, 12,14,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lcolord_w, 14,17,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lcolor_w, 17,19,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), sl_w, 1,4,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _sstyle_w, 4,8,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), sth_w, 8,10,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _sthick_w, 10,12,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), sc_w, 12,14,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _scolord_w, 14,17,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _scolor_w, 17,19,4,5);

   gtk_table_attach_defaults(GTK_TABLE(table), fl_w, 1,4,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _fstyle_w, 4,8,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), fo_w, 8,10,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _fopacity_w, 10,12,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), fc_w, 12,14,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _fcolord_w, 14,17,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _fcolor_w, 17,19,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _dvis_w, 10,13,6,7);

   gtk_table_attach_defaults(GTK_TABLE(table), bh_w, 1,5,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), _oplot_w, 6,10,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), _nframe_w, 10,14,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), _erase_w, 14,19,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), label_w, 1,5,9,10);
   gtk_table_attach_defaults(GTK_TABLE(table), cb_w, 6,14,9,10);
   gtk_table_attach_defaults(GTK_TABLE(table), _launch, 15,19,9,10);
   gtk_table_attach_defaults(GTK_TABLE(table), lab2_w, 1,5,10,11);
   gtk_table_attach_defaults(GTK_TABLE(table), _shell_no_w, 6,9,10,11);
   gtk_table_attach_defaults(GTK_TABLE(table), _shell_py_w, 10,13,10,11);

   gtk_widget_show_all(_scrolled_w);

   g_signal_connect(_cvx_w,"changed",G_CALLBACK(changed_value_callback),
                    (void*)this);
   validate(); 
}

int PrismHistogramCmd::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismHistogramCmd::doit(PrismDataSet* dset, string usecol, int dialog)
{
   bool use_dialog = false;

   if (!theApp) return 1;       // app does not exist 
   
   if (dialog == -1)
   {
      if (theApp->get_preference("data.histo.dialog", use_dialog))  
      use_dialog = true;
   }
   else if (dialog == 1)
      use_dialog = true; 
   
   if ((_dset = dset) == NULL) 
   {
      theApp->status_message("No data to histogram in current tab"); 
      return 1; 
   }

   // make sure enough columns are specified for quick histogram
   if (!use_dialog && usecol.length() == 0)
   {
      theApp->status_message("Unable to generate Histogram. Please select at least 1 column (0 selected).", 1);
      return 1; 
   } 

   // keep last value if a blank is passed in 
   if (usecol.length())  _xcol = usecol; 

   if (use_dialog)
   {
      // generate a dialog to allow user to specify defaults 
      generate_form(); 

      int resp = GTK_RESPONSE_APPLY;
      while (resp == GTK_RESPONSE_APPLY)
      {
         resp = gtk_dialog_run(GTK_DIALOG(base_w));
         if (resp == GTK_RESPONSE_APPLY || resp == GTK_RESPONSE_OK)
         {
            GdkColor clr;
            char hexcolor[16]; 
	    if (_entry)  _reqId = gtk_entry_get_text(_entry);
            if (_launch) 
	       _instantiate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_launch));
	    // set behavior mode 
	    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_oplot_w)))
               _behavior = 0;
            else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_nframe_w))) 
               _behavior = 1;
	    else 
	       _behavior = 2; 

            // get cols to plot
            GtkEntry* entx_w = GTK_ENTRY(GTK_BIN(_cvx_w)->child);
            _xcol = gtk_entry_get_text(GTK_ENTRY(entx_w));
	    _autobin = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_autobin_w));
            _bins = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_cvy_w));
   
	    _lstyle = gtk_combo_box_get_active(GTK_COMBO_BOX(_lstyle_w));
	    _sstyle = gtk_combo_box_get_active(GTK_COMBO_BOX(_sstyle_w));

            _lthick = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_lthick_w));
            _sthick = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_sthick_w));
            _fopacity = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_fopacity_w));
	 
            _dvis = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_dvis_w));
	    _fstyle = gtk_combo_box_get_active(GTK_COMBO_BOX(_fstyle_w));
   
            _shell_py=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_shell_py_w));

            if (!(_lcolord=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_lcolord_w))))
	    {
	       gtk_color_button_get_color(GTK_COLOR_BUTTON(_lcolor_w), &clr);
               sprintf(hexcolor,"#%02x%02x%02x",clr.red/256,clr.green/256,clr.blue/256); 
	       _lcolor = hexcolor; 
	    }
            if (!(_scolord=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scolord_w))))
	    {
	       gtk_color_button_get_color(GTK_COLOR_BUTTON(_scolor_w), &clr);
	       sprintf(hexcolor,"#%02x%02x%02x",clr.red/256,clr.green/256,clr.blue/256); 
	       _scolor = hexcolor;
            }
            if (!(_fcolord=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_fcolord_w))))
	    {
	       gtk_color_button_get_color(GTK_COLOR_BUTTON(_fcolor_w), &clr);
	       sprintf(hexcolor,"#%02x%02x%02x",clr.red/256,clr.green/256,clr.blue/256); 
	       _fcolor = hexcolor;
            }
            generate_plot(); 
         }
      }
      gtk_widget_destroy(base_w); 
   }
   else
   {
      // skip dialog and just generate an image using defaults
      _reqId = _chipsId; 
      _shell_py = false; 

      set_default_column_name();

      if (_xcol.length() && _collist.find(";"+_xcol+";") == string::npos)
      {
         theApp->status_message("Unable to generate histogram. Please verify that column " + _xcol + " is not a vector or array column.", 1);
	 return 1; 
      }
      generate_plot(); 
   }
   return 0;
}

int PrismHistogramCmd::generate_plot()
{
   char props[2048]; 
   int  zz; 
   string filespec, extnam, axlab = "";
   PyObject *py_retval = NULL;

   if (!_dset)  return 2;   // don't have a data file 

   filespec = _dset->get_filename();
   extnam = _dset->get_current_extname(); 
   if (extnam.length())  filespec.append("["+extnam+"]"); 

   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   theApp->connect_plotter(_reqId);

   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   _chipsId = theApp->get_chipsId(); 

   if (_shell_py )
   {
      char namebuf[256];
      int pd = (int)getpid();
      char* lgname = getlogin();
      if (lgname)  sprintf(namebuf,"/tmp/launch_chips.%s.%1d", lgname, pd);
         else      sprintf(namebuf,"/tmp/launch_chips.%1d", pd);
      ofstream out(namebuf);
      out << "connect(\"" << _chipsId << "\");" << endl;
      out.close();
      string cmd = "ASCDS_RUN_CONFIG=: chips -x ";
      cmd.append(namebuf);
      cmd.append(" &"); 
      system(cmd.c_str()); 
   }

   if (_xcol.length())
   {
      // remove existing [cols spec if present
      string::size_type p1, p2;
      if ((p1 = filespec.find("[cols ")) != string::npos)
      {
         if ((p2 = filespec.find("]", p1)) != string::npos)
         {
	    string tmp = filespec.substr(0, p1+6) + _xcol + filespec.substr(p2);
	    filespec = tmp;
         }
      }
      else
      {
         filespec.append("[cols ");
         filespec.append(_xcol);
         filespec.append("]");
      }
      // setup axis labels- replacing underscores to avoid latex subscripting
      axlab = "axis.y.label=counts axis.x.label=";
      for (unsigned int ll = 0; ll < _xcol.length(); ll++)
         if (_xcol[ll] == '_') axlab.append("\\_");  
            else  axlab += _xcol[ll]; 
   }

   // refresh gui- since widgets in sheet seem to hang g_main_context_iter, add
   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   if ((py_retval = theApp->call_py_functions(NULL, "open_undo_block")) != NULL)
      Py_DECREF(py_retval);

   if (_behavior == 1)
   {
      if ((py_retval = theApp->call_py_functions(NULL, "add_window")) != NULL)
         Py_DECREF(py_retval);
   }
   else if (_behavior == 2) 
   {
      if ((py_retval = theApp->call_py_functions(NULL, "erase")) != NULL)
         Py_DECREF(py_retval);
   }

   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   // load attributes
   if (_lcolord || _lcolor.length()) 
   {
      axlab.append(" line.color="); 
      if (_lcolord)   axlab.append("default");
      else
      {
	 if (_lcolor[0] == '#')
	    axlab.append(_lcolor.substr(1, _lcolor.length()-1));
	 else                   
            axlab.append(_lcolor);
      }
   }
   if (_scolord || _scolor.length())
   {
      axlab.append(" symbol.color="); 
      if (_scolord)   axlab.append("default");
      else
      {
	 if (_scolor[0] == '#')
	    axlab.append(_scolor.substr(1, _scolor.length()-1));
	 else                   
            axlab.append(_scolor);
      }
   }
   if (_fcolord || _fcolor.length())
   {
      axlab.append(" fill.color="); 
      if (_fcolord)   axlab.append("default");
      else
      {
	 if (_fcolor[0] == '#')
	    axlab.append(_fcolor.substr(1, _fcolor.length()-1));
	 else                   
            axlab.append(_fcolor);
      }
   }
   sprintf(props, "line.style=%d symbol.style=%d line.thickness=%1.1f symbol.size=%d dropline=%1d fill.style=%1d fill.opacity=%2.1f %s", 
      _lstyle, _sstyle, _lthick, _sthick, _dvis, _fstyle,_fopacity,axlab.c_str()); 

   if (_autobin)
   {
      py_retval = theApp->call_py_functions(NULL, "p2hGenerateHistogram", 
         "%s %p %p %p %p %s", filespec.c_str(), Py_None, Py_None, Py_None, 
	 Py_None, props);
   }
   else
   {
      py_retval = theApp->call_py_functions(NULL, "p2hGenerateHistogram", 
         "%s %d %p %p %p %s", filespec.c_str(), _bins, Py_None, Py_None, 
	 Py_None, props);
   }

   if (!py_retval)
   {
      theApp->status_message("Failed to add histogram of " + filespec + " to chips server '"+
         _chipsId + "'", 1);
      return 1; 
   }
   else
      theApp->status_message("Added histogram of " + filespec + " to chips server '"+
         _chipsId + "'");
   Py_DECREF(py_retval);

   if ((py_retval = theApp->call_py_functions(NULL, "close_undo_block")) != NULL)
      Py_DECREF(py_retval);

   return 0;
}


int PrismHistogramCmd::autobin_toggle_callback(GObject *object, gpointer user_data)
{
   PrismHistogramCmd* cmd = (PrismHistogramCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->set_bin_button_sensitive();
   return 0;
}

int PrismHistogramCmd::toggled_value_callback(GObject *object, gpointer user_data)
{
   PrismHistogramCmd* cmd = (PrismHistogramCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->set_color_buttons_sensitive();
   return 0;
}

void PrismHistogramCmd::set_color_buttons_sensitive()
{
   bool _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_lcolord_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_lcolor_w), !_toggle);
   _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scolord_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_scolor_w), !_toggle);
   _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_fcolord_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_fcolor_w), !_toggle);
}


void PrismHistogramCmd::set_bin_button_sensitive()
{
   bool _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_autobin_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_cvy_w), !_toggle);
}


void PrismHistogramCmd::set_default_column_name(bool dialog)
{
   PrismDataBlock *bl = _dset->get_block();
   if (bl)
   {
      PrismDataColumn *dcols = bl->get_columns();
      long cc, rr, found = 0, cnt = bl->get_totcols();
      string cname;

      _collist = ";"; 
      for (cc = rr = 0; rr < cnt; rr++)
         if (!(dcols[rr].is_array() || dcols[rr].is_array_comp() ||
	     dcols[rr].is_vector()))
         {
            cname = dcols[rr].get_name();
	    _collist.append(cname+";");
	    if (dialog)
               gtk_combo_box_append_text(GTK_COMBO_BOX(_cvx_w), cname.c_str());
            if (!found && cname == _xcol)
	    { 
               if (dialog)  gtk_combo_box_set_active(GTK_COMBO_BOX(_cvx_w), cc);
	       found = 1; 
	    }
            cc++;
         }
   }
}

void PrismHistogramCmd::validate()
{
   bool valid =  false;
   if (_cvx_w)
   {
      GtkEntry* entx_w = GTK_ENTRY(GTK_BIN(_cvx_w)->child);
      string xx = gtk_entry_get_text(GTK_ENTRY(entx_w));
      if (xx.length() == 0)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify a value for Column");
      else if (_collist.find(";"+xx+";") == string::npos)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify a column name from the pulldown options");
      else
      {
         gtk_label_set_text(GTK_LABEL(_status_w), "");
         valid = true;
      }
   }
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_OK,valid);
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_APPLY, valid);
}


int PrismHistogramCmd::changed_value_callback(GObject *object, gpointer user_data)
{
   PrismHistogramCmd* cmd = (PrismHistogramCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->validate();
   return 0;
}



