// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMHISTOGRAMCMD_HH
#define PRISMHISTOGRAMCMD_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>
#include <list>

#include <Python.h>

#include "PrismBaseObj.hh"

using namespace std;

class PrismDataSet;

class PrismHistogramCmd: public PrismBaseObj
{
  public:
    PrismHistogramCmd();
    ~PrismHistogramCmd();
    virtual int show();

    int doit(PrismDataSet* dset, string cname="",
             int dialog=-1); // -1=app default, 0=no 1=yes
    int generate_plot();

    static int autobin_toggle_callback(GObject *object, gpointer user_data);
    static int toggled_value_callback(GObject *object, gpointer user_data);
    static int changed_value_callback(GObject *object, gpointer user_data);

    void set_bin_button_sensitive(); 
    void set_color_buttons_sensitive(); 
    void validate(); 

  protected:
    void generate_form(); 
    void set_default_column_name(bool dialog=false);

    GtkEntry*     _entry;         // entry widget from combo pulldown 
    GtkWidget*    _launch;        // toggle option to launch ds9 
    GtkWidget*    _shell_no_w;    // no interactive shell toggle 
    GtkWidget*    _shell_py_w;    // python interactive shell toggle 
    GtkWidget*    _scrolled_w;    // scrolled window widget         
    GtkWidget*    _vbox_w;        
    GtkWidget*    _oplot_w;       // add curve to existing plot (overplot) 
    GtkWidget*    _nframe_w;      // add curve in new frame 
    GtkWidget*    _erase_w;       // erase all windows and create new one

    GtkWidget*    _cvx_w;         // curve x entry
    GtkWidget*    _cvy_w;         // curve y entry
    GtkWidget*    _cve_w;         // curve e entry

    GtkWidget*    _lstyle_w;      // line style 
    GtkWidget*    _lcolor_w;      // line color 
    GtkWidget*    _lcolord_w;     // color default 
    GtkWidget*    _lthick_w;      // line thickness 
    GtkWidget*    _sstyle_w;      // symbol style 
    GtkWidget*    _scolor_w;      // symbol color 
    GtkWidget*    _scolord_w;     // color default 
    GtkWidget*    _sthick_w;      // symbol thickness 
    GtkWidget*    _fcolor_w;      // fill color 
    GtkWidget*    _fcolord_w;     // fill color default 
    GtkWidget*    _fopacity_w;    // fill opacity
    GtkWidget*    _fstyle_w;      // fill visibility 
    GtkWidget*    _dvis_w;        // fill visibility 
    GtkWidget*    _autobin_w;     // ato calc bins or use specified?
    GtkWidget*    _frm_w;         // frame aorund plot attributes
    GtkWidget*    _status_w;      // status label

    PrismDataSet* _dset; 
    bool          _instantiate; 
    bool          _lcolord;       // line color default
    bool          _scolord;       // symbol color default
    bool          _fcolord;       // fill color default
    bool          _dvis;          // dropline visible
    bool          _shell_py;      // launch python shell 
    bool          _autobin;       // auto calculate bins  

    string        _chipsId;       // current chips Server id
    string        _reqId;         // rquested chips Server id
    string        _xcol;          // x column 
    string        _collist;       // list of possible columns

    int           _bins;          // number of bins 
    int           _lstyle;        // line style
    int           _fstyle;        // fill style
    string        _lcolor;        // line color
    double        _lthick;        // line thickness (.5:10)
    int           _sstyle;        // symbol style 
    string        _scolor;        // symbol color
    int           _sthick;        // symbol size (1:100)
    string        _fcolor;        // fill color 

    float         _fopacity;      // fill opacity 

    int           _behavior;      // overplot, add frame, or erase?
  
    list<string>  _plotters;      // list of imagers instantiated 

};

#endif 
