// 
//  Copyright (C) 2008,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMPREFSDIALOG_HH
#define PRISMPREFSDIALOG_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <gdk/gdk.h>
#include <string>

#include "PrismBaseObj.hh"

using namespace std;

class PrismPrefs;

class PrismPrefsDialog: public PrismBaseObj
{
  public:
    PrismPrefsDialog(PrismPrefs *prefs);
    ~PrismPrefsDialog();
    void doit(); 

  protected:
    virtual int show();
    void generate_form(); 

    GtkWidget* _scrolled; 
    GtkWidget* _vbox; 
    GtkWidget* _tips_w;

    GtkWidget* _conf_dlg_w;
    GtkWidget* _hdr_exp;
    GtkWidget* _hdr_table;
    GtkWidget* _hdr_raw_w;
    GtkWidget* _disp_name_w; 
    GtkWidget* _disp_value_w; 
    GtkWidget* _disp_type_w; 
    GtkWidget* _disp_cmt_w; 
    GtkWidget* _disp_units_w; 
    GtkWidget* _disp_keyno_w; 
    GtkWidget* _hdr_add_dlg_w; 
    GtkWidget* _hdr_del_dlg_w; 

    GtkWidget* _data_exp;
    GtkWidget* _data_table;
    GtkWidget* _data_plot_dlg_w;
    GtkWidget* _data_hist_dlg_w;
    GtkWidget* _data_img_dlg_w;
    GtkWidget* _data_rows_w; 
    GtkWidget* _exp_vector_w;
    GtkWidget* _clp_vector_w;
    GtkWidget* _exp_array_w;
    GtkWidget* _clp_array_w;
    GtkWidget* _exp_ve_w; 
    GtkWidget* _clp_ve_w; 
    GtkWidget* _exp_ae_w; 
    GtkWidget* _clp_ae_w; 

    GtkWidget* _selectfg_w; 
    GtkWidget* _selectbg_w; 

    GtkWidget* _info_exp;
    GtkWidget* _info_table;
    GtkWidget* _info_desc_w;
    GtkWidget* _info_ctype_w;
    GtkWidget* _info_dtype_w;
    GtkWidget* _info_units_w;
    GtkWidget* _info_min_w;
    GtkWidget* _info_max_w;
    GtkWidget* _info_null_w;
    GtkWidget* _info_exist_w;
    GtkWidget* _info_coord_w;
    GtkWidget* _info_crpix_w;
    GtkWidget* _info_crval_w;
    GtkWidget* _info_cdelt_w;

    void load_prefs(); 
    void load_dialog_prefs(); 
    void retrieve_dialog_prefs();
    void commit_prefs(); 
    void save_prefs(); 

  private:
    PrismPrefs* _prefs; 

    static bool _hdrexp;      // expand header entries - track instances 
    static bool _datexp;      // expand data entries - track instances 
    static bool _infoexp;     // expand info entries - track instances 

    long _rowspp;      // nubmer of rows per page

    bool _ttips;       // show tool tips 
    bool _conf;        // edit confirmation dialog
    bool _hdrraw;      // filter raw header keywords
    bool _hdrname;     // display header name
    bool _hdrvalue;    // display header value
    bool _hdrtype;     // display header type 
    bool _hdrcomment;  // display header comment 
    bool _hdrunit;     // display header unit 
    bool _hdrkeyno;    // display header keyno 
    bool _hdradd;      // add dialog from toolbar button?
    bool _hdrdel;      // delete dialog from toolbar button?
    bool _datplot;     // plot dialog from toolbar button?
    bool _dathist;     // histogram dialog from toolbar button?
    bool _datimg;      // image dialog from toolbar button?
    bool _expv;        // expand vector 
    bool _expa;        // expand array 

    bool _info_desc;   // display info data description 
    bool _info_ctype;  // display info data column type 
    bool _info_dtype;  // display info data type 
    bool _info_units;  // display info data units 
    bool _info_min;    // display info data range min 
    bool _info_max;    // display info data range max 
    bool _info_null;   // display info data tnull 
    bool _info_exists; // display info data physical vs virtual 
    bool _info_coord;  // display info data coord sys 
    bool _info_crpix;  // display info data crpix 
    bool _info_crval;  // display info data crval 
    bool _info_cdelt;  // display info data cdelt 

    GdkColor _prevfg;  // previous foreground select color
    GdkColor _prevbg;  // previous background select color
    string _expv_filt; // expand vector filter 
    string _expa_filt; // expand array filter 
    string _clpv_filt; // collapse vector filter 
    string _clpa_filt; // collapse array filter 
    string _selectfg;  // select foreground color 
    string _selectbg;  // select foreground color 
};

#endif 
