// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismStatusArea.hh"

#include <iostream>

using namespace std;

PrismStatusArea::PrismStatusArea():PrismBaseObj()
{
   base_w = gtk_scrolled_window_new(NULL, NULL);
        gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (base_w),
                                       GTK_POLICY_ALWAYS,
                                       GTK_POLICY_ALWAYS);
   buffer = gtk_text_buffer_new(NULL);
   textBox = gtk_text_view_new_with_buffer(buffer);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(textBox), false);
   gtk_container_add (GTK_CONTAINER (base_w), textBox);
   prepend = true; 
   cnt = 0; 
   color_tag = gtk_text_buffer_create_tag (buffer, NULL,
      	   		            "foreground", "red", NULL);  
   bold_tag = gtk_text_buffer_create_tag (buffer, "bold",
				    "weight", PANGO_WEIGHT_BOLD, NULL);
   italic_tag = gtk_text_buffer_create_tag (buffer, "italic",
				    "style", PANGO_STYLE_ITALIC, NULL);
   use_bold = false; 
   use_color = true; 
   use_italic = false;
   color = "red"; 
   mark = NULL; 
}

PrismStatusArea::~PrismStatusArea()
{
   
}


int PrismStatusArea::add_message(string msg, int type)
{
   if (msg.length())
   {
      // add timestamp to message- although it could be done using glib's
      // time functions- use the standard c libs for now...
      static int ln;
      time_t secs;
      char tstamp[33];
      string str;
      time(&secs);
      strftime(((char*)tstamp)+1,32,"%a %d-%b %H:%M:%S ",localtime(&secs));
      str = tstamp+1;

      GtkTextIter iter, iter2;
      if (prepend)
         gtk_text_buffer_get_start_iter (buffer, &iter);
      else 
         gtk_text_buffer_get_iter_at_line (buffer, &iter, ln++);
      gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer), &iter, str.c_str(), -1);

      if (mark)
         gtk_text_buffer_move_mark_by_name(GTK_TEXT_BUFFER(buffer),"mark",&iter);
      else 
         mark = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(buffer), "mark",
	    &iter,true);
      gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer), &iter, msg.c_str(), -1);

      if (type == 1)
      {
         gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(buffer), &iter2, mark); 
	 if (use_color) gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(buffer),
	    color_tag, &iter2, &iter);
	 if (use_bold) gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(buffer),
	    bold_tag, &iter2, &iter);
	 if (use_italic) gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(buffer),
	    italic_tag, &iter2, &iter);
      }
      gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer), &iter, "\n", -1);
      cnt++; 
   }
   return 0; 
}

int PrismStatusArea::show()
{
   gtk_widget_show(base_w);
   return 0;
}


string PrismStatusArea::get_all_messages()
{
   buff.clear();
   GtkTextIter start_i, stop_i;
   gtk_text_buffer_get_start_iter (buffer, &start_i);
   gtk_text_buffer_get_end_iter (buffer, &stop_i);
   buff = gtk_text_buffer_get_text(buffer, &start_i, &stop_i, false); 
   return buff; 
}

void PrismStatusArea::set_error_color(string val)
{
   GdkColor clr;
   if (val.length() && gdk_color_parse(val.c_str(), &clr))
   {
      color = val; 
      color_tag = gtk_text_buffer_create_tag(buffer, NULL, "foreground",
      	   		                     color.c_str(), NULL);  
   }
}   

