// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMTABLECOLUMN_HH
#define PRISMTABLECOLUMN_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <gtkextra/gtksheet.h>
#include <iostream>
#include <string>

class PrismDataColumn; 

#define PRISM_COLSTATE_NO_LOAD    0
#define PRISM_COLSTATE_PRELOAD    1
#define PRISM_COLSTATE_SKIP       2
#define PRISM_COLSTATE_MANLOAD    3 

using namespace std;

class PrismTableColumn
{
   public:
      PrismTableColumn(PrismDataColumn* col, long colmin, long colmax=-1, 
                       int state= PRISM_COLSTATE_PRELOAD);
      ~PrismTableColumn(); 
      int    get_id()        { return _id;  } 
      long   get_colmin()    { return _colmin; }
      long   get_colmax()    { return _colmax; }
      int    get_colminmax(long &tmin, long &tmax); 
      void   set_colminmax(long tmin, long tmax) 
                             { _colmin = tmin; _colmax = tmax; }
      long   get_togmin()    { return _togmin; }
      long   get_togmax()    { return _togmax; }
      int    get_togminmax(long &tmin, long &tmax); 
      void   set_togminmax(long tmin, long tmax) 
                             { _togmin = tmin; _togmax = tmax; }
      void   set_state(int val)   { _state = val; }

      PrismDataColumn* get_data_column()  { return _dcol; }
      int    get_state()     { return _state; } 
      bool   is_image()      { return _image; }
      void   is_image(bool val)  { _image = val; }

   private:
      PrismDataColumn * _dcol;
      long    _colmin;     // min table column for this entry 
      long    _colmax;     // max table column for this entry  
      long    _togmin;     // min table column for array|vector toggling
      long    _togmax;     // max table column for array|vector toggling 
      int     _state;      // 0 = not loaded, 1 = preloaded, 2 = skipped, 3 = man loaded
      int     _err;        // 0 = good  
      int     _id; 
      bool    _image;      // column is image column (rather than table data)
      static int _index;   // sequential index for ids 
}; 

#endif 
