// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismApp.hh"
#include "PrismViewMenu.hh"

using namespace std;

PrismViewMenu::PrismViewMenu(): PrismBaseObj()
{
   base_w = gtk_menu_item_new_with_mnemonic("_View");

   menu = gtk_menu_new();

   menuDispTabs = gtk_menu_item_new_with_mnemonic("_Display Tabs");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuDispTabs, 0);
   menuHideTabs = gtk_menu_item_new_with_mnemonic("_Hide Tabs");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuHideTabs, 1);
   menuPosTabs = gtk_menu_item_new_with_label("Position Tabs");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuPosTabs, 2);

   subMenuPos = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuPosTabs), subMenuPos);
   menuPosTop = gtk_menu_item_new_with_label("Top");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subMenuPos), menuPosTop, 0);
   menuPosBottom = gtk_menu_item_new_with_label("Bottom");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subMenuPos), menuPosBottom, 1);
   menuPosLeft = gtk_menu_item_new_with_label("Left");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subMenuPos), menuPosLeft, 2);
   menuPosRight = gtk_menu_item_new_with_label("Right");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subMenuPos), menuPosRight, 3);
   g_signal_connect_swapped (G_OBJECT(menuPosLeft), "activate",
                             G_CALLBACK(position_tabs_cb), (gpointer) 0);
   g_signal_connect_swapped (G_OBJECT(menuPosRight), "activate",
                             G_CALLBACK(position_tabs_cb), (gpointer) 1);
   g_signal_connect_swapped (G_OBJECT(menuPosTop), "activate",
                             G_CALLBACK(position_tabs_cb), (gpointer) 2);
   g_signal_connect_swapped (G_OBJECT(menuPosBottom), "activate",
                             G_CALLBACK(position_tabs_cb), (gpointer) 3);

   GtkTooltips* tips = theApp->get_tips_group();
   if (tips)
   {
      gtk_tooltips_set_tip(tips, menuDispTabs,
         "display tabs containing file ids",NULL);
      gtk_tooltips_set_tip(tips, menuHideTabs,
         "hide tabs containing file ids",NULL);
      gtk_tooltips_set_tip(tips, menuPosTabs,
         "change the location of where tabs are displayed",NULL);
      gtk_tooltips_set_tip(tips, menuPosTop,
         "display tabs on the top",NULL);
      gtk_tooltips_set_tip(tips, menuPosRight,
         "display tabs on the right",NULL);
      gtk_tooltips_set_tip(tips, menuPosBottom,
         "display tabs on the bottom",NULL);
      gtk_tooltips_set_tip(tips, menuPosLeft,
         "display tabs on the left",NULL);
   }

   g_signal_connect_swapped (G_OBJECT (menuDispTabs), "activate",
                             G_CALLBACK (display_tabs_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuHideTabs), "activate",
                             G_CALLBACK (hide_tabs_cb),
                             (gpointer) NULL);

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

PrismViewMenu::~PrismViewMenu()
{
   
}

int PrismViewMenu::show()
{
   gtk_widget_show(menuDispTabs);
   gtk_widget_show(menuHideTabs);
   gtk_widget_show(menuPosTabs);
   gtk_widget_show(base_w); 
   return 0; 
}

int PrismViewMenu::display_tabs_cb()
{
   theApp->show_tabs(); 
   return 0; 
}

int PrismViewMenu::hide_tabs_cb()
{
   theApp->hide_tabs(); 
   return 0; 
}

int PrismViewMenu::position_tabs_cb(gpointer user_data)
{
   gint pos = GPOINTER_TO_INT(user_data); 
   if (pos < 0 || pos > 3)  return 1; 
   theApp->reposition_tabs(pos); 
   return 0; 
}


