// 
//  Copyright (C) 2007,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef TASKMONAPP_HH
#define TASKMONAPP_HH

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <iostream>
#include <string>

class TaskmonTextArea;
class TaskmonCmdButtons; 
class TaskmonStatusArea;
class TaskmonMenu;
class CiaoGuiIPCServer;
class CiaoGuiIPCMessage; 

using namespace std;


class TaskmonApp
{
  public:

    TaskmonApp();
    ~TaskmonApp();

    #define MAX_PIPE_SIZE    5120
    #define TASKM_UNKNOWN_STS           32767
    #define TASKM_UNKNOWN_STS_STR      "32767"

    string _cmdline;     // string containing the command line arguments.
    string     GetCmdline() { return _cmdline; }

    static void exit_cb();

    void exit_application(); 

    string get_file_path(); 

    int command_message(string msg);
    int output_message(string msg);
    int status_message(string msg);

    void initialize(string toolfile="", string xpaname=""); 

    GtkTooltips* get_tips_group()  { return _tips; }

    void set_window_title(string val); 

    GtkWidget*  get_command_scrolled()     { return _commandscrolled; }
    GtkWidget*  get_output_scrolled()     { return _outputscrolled; }

    TaskmonCmdButtons* getCmdButtons() { return _buttons;  }

  // PIPE and XPA declarations
    int _pid; 
    int _finished; 
    int _status; 
    int pipefd1[2];
    int          GetPid()  { return _pid; }
    int          GetPipeFD0() { return pipefd1[0]; }
    int          GetPipeFD1() { return pipefd1[1];}
    int          ProcessFinished() { return _finished; } 
    void         SetFinished(int vv) { _finished = vv; } 
    int          GetStatus()         { return _status; }
    void         SetStatus(int val)  { _status = val; } 
    string       GetXpaName()        { return _xpaname; }
    bool      Find(string str, bool caseflag, bool startflag);

    void PipeSetup();
    static gboolean handle_command_output(GIOChannel *gio, 
					  GIOCondition condition, 
					  gpointer data);

  //Text buffers and command buttons in the gui
    TaskmonTextArea* _commandArea;
    TaskmonTextArea* _outputArea;
    TaskmonCmdButtons* _buttons; 
    TaskmonStatusArea* _status_area;


  protected: 
    void cleanup(); 
    void reset_tips(); 


    void   add_ipc();
    CiaoGuiIPCServer* ipc;
    static CiaoGuiIPCMessage* CommandMsgCB(CiaoGuiIPCMessage& msg);
    static CiaoGuiIPCMessage* FindMsgCB(CiaoGuiIPCMessage& msg);
    static CiaoGuiIPCMessage* StatusMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* ToolStatusMsgCB(CiaoGuiIPCMessage&);

  private:
    GtkWidget* _window; 

    GtkWidget* _mainvbox;    // holds command and output panes; status bar and
                             // buttons

    GtkWidget* _topframe;          //top frame for the title "Command"
    GtkWidget* _commandscrolled;  // scrolled window for command area 

    GtkWidget* _bottomframe;       //bottom frame for the title "output"
    GtkWidget* _outputscrolled;   // scrolled window for output area

    GtkTooltips *_tips;      // tool tips group

    TaskmonMenu*   _topmenu;

    static string statusId;  // unique identifier for status message 
    string     _xpaname;     // xpa access name
    int        _height;      // initial height 
    int        _width;       // initial width  
    int        _init;        // has taskmonitor been initialized?
    bool       _showtips;    // display tooltips 
};

extern TaskmonApp *theApp;

#endif 

