// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonFileDialog.hh"
#include <iostream>
#include <fstream>
#include <unistd.h>
#include <gtk/gtk.h>

using namespace std;

TaskmonFileDialog::TaskmonFileDialog()
{

} //End Constructor

TaskmonFileDialog::~TaskmonFileDialog()
{
   if (_filedialog)
    {
      gtk_widget_destroy (_filedialog);
      _filedialog = NULL;
    }
} // end Destructor

int TaskmonFileDialog::_pop_it_up()
{
  string buffers;
  gint result;
  GtkStockItem filter_item;
  ofstream outfile;
  string buf_contents;
  int status=0;
  int writeflag = 0;

  // This dialog box has two sections: 
  //         An output selection ( a label and 2 check buttons)
  //         The File selection wdget



  // There is no handy STOCK item for "Filter" So use the INDEX
  // stock item and change it's label to Filter

  if(gtk_stock_lookup( (gchar*)"gtk-index", &filter_item) )
    {
     filter_item.label = "_Filter";
     gtk_stock_add (&filter_item, 1);
    }
  else
    cout<<endl<<"stock lookup failed"<<endl;

  // create the starting dialog box container
   _dialog = gtk_dialog_new_with_buttons("Save As", NULL,
					 GTK_DIALOG_DESTROY_WITH_PARENT, 
			                 GTK_STOCK_OK, GTK_RESPONSE_OK, 
					 GTK_STOCK_CLEAR, GTK_RESPONSE_ACCEPT,
				      	 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					 NULL);

 // Section 1 - Output section. Make it a 4x8 table and pack the 
 // label and two check buttons in a row

   // create the 3x8 table
   _output_table = gtk_table_new(4,8, true);

   //create the label and pack it in row 2, left of the table
   _output_label = gtk_label_new("Output:"); 
   gtk_table_attach_defaults(GTK_TABLE(_output_table), _output_label, 0,2,1,2);

   // Now create two check buttons and pack them in row 3, #2 left
   _output_commandCHKB = gtk_check_button_new_with_label ("Command");
   gtk_toggle_button_set_active((GtkToggleButton*)_output_commandCHKB, TRUE);
   gtk_table_attach_defaults(GTK_TABLE(_output_table),
			     _output_commandCHKB,
			     2,4,
			     2,3);

   _output_resultsCHKB = gtk_check_button_new_with_label ("Results");
   gtk_toggle_button_set_active((GtkToggleButton*)_output_resultsCHKB, TRUE);
   gtk_table_attach_defaults(GTK_TABLE(_output_table),
			     _output_resultsCHKB,
			     5,7,
			     2,3);

   // Now pack the whole Section one into the dialog vbox
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), _output_table);


 // Pack a separator in between section 1 and section 2

   _sep1 = gtk_hseparator_new();
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), _sep1);
 


  /* Create the widgets */
   _filedialog = gtk_file_chooser_widget_new (GTK_FILE_CHOOSER_ACTION_SAVE );
   gtk_file_chooser_set_show_hidden((GtkFileChooser*)_filedialog, TRUE);
	    

   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), _filedialog );

   // Show the dialog. 
    gtk_widget_show_all (_dialog);

   //
   // PROCESSING
   //
   // At least ONE of the check boxes up top have to be set
   // check them. and if one is not set, pop up a message dialog 
   // and stay in the while loop until one is set

    // first wait for a button click and then
    // gather the information on the buttons and File selection
    // lastly set the write flag = 0 meaning no file write has occurred

     result = gtk_dialog_run (GTK_DIALOG (_dialog));
     status = _gather_info();
     writeflag = 0;

    // While the user didn't hit **CANCEL**, AND a writeout has not occurred

     while ( (result != GTK_RESPONSE_CANCEL) &&
	     (writeflag == 0) )
       {
	 // if the user hit **CLEAR**
          if(result == GTK_RESPONSE_ACCEPT)
           {
	    gtk_file_chooser_set_current_name((GtkFileChooser*)_filedialog, "");
            // get another button click because after the clear, the
	    // user might then hit OK, or even CANCEL
            result = gtk_dialog_run (GTK_DIALOG (_dialog));
            status = _gather_info();
           }// end while(result == GTK_RESPONSE_ACCEPT)

	  //otherwise check the values of the checkbuttons
          else if( (_o_cCHKBval == 0) && 
	           (_o_rCHKBval == 0) )
	         {
	          //create and pop up the error dialog informing the user, 
	          // then destroy
	          TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
	          _error_dialog->_pop_error_up("You must have at least one, or both, of the checkboxes checked indicating what text field you wish to have written to the file.");
	          delete _error_dialog;
                 // Drop into the blocking loop and await a user click on
                 // one of the buttons at the bottom
                  result = gtk_dialog_run (GTK_DIALOG (_dialog));
                  status = _gather_info();
		 } // end if( (<_o_cCHKBval == 0) && (_o_rCHKBval == 0) )

	  // otherwise check to see if the file path was given
	  else if (filespec == NULL)
	        {
	         //create and pop up the error dialog informing the user, 
	         // then destroy
	          TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
	          _error_dialog->_pop_error_up("You must specify a file into which to write the data!");
	         delete _error_dialog;
                // Drop into the blocking loop and await a user click on
                // one of the buttons at the bottom
                 result = gtk_dialog_run (GTK_DIALOG (_dialog));
                 status = _gather_info();
	        } // end if( (filespec == NULL )

	  else //else it wasn't CANCEL, wasn't CLEAR, and all fields correct
	       // so now write one or both of the buffers contents to the file
               // based upon the values of the check buttons
	    {
             // Open the file
             outfile.open(filespec, ios_base::out);
	       if (outfile)    // output file sucessfully opened
	        {
                  if ( _o_cCHKBval == 1)
                   {
                    outfile << "\n COMMAND : \n --------- \n" << endl;
	            buf_contents = theApp->_commandArea->get_buffer_contents();
                    outfile << buf_contents << endl;
                   }
	   
                  if( _o_rCHKBval == 1)
                   {
                    outfile << "\n RESULTS : \n --------- \n" << endl;
	            buf_contents = theApp->_outputArea->get_buffer_contents();
                    outfile << buf_contents << endl;
                   }
                 outfile.close();
	         writeflag = 1;
		}// end if (outfile)
            else  //else the file open failed...bail
             cerr<<endl<<"ERROR - file open failed"<<endl;

	    }// end write out the data to the file

       } // end while (result != GTK_RESPONSE_CANCEL) and writeflag = 0

       gtk_widget_destroy (_dialog);
            _dialog = NULL;

   return(0);


} // end method _pop_it_up()


int TaskmonFileDialog::_gather_info()
{
      // Gather the check button values
      _o_cCHKBval = gtk_toggle_button_get_active((GtkToggleButton*)_output_commandCHKB);
      _o_rCHKBval = gtk_toggle_button_get_active((GtkToggleButton*)_output_resultsCHKB);

      filespec = gtk_file_chooser_get_filename((GtkFileChooser*)_filedialog);


  return(0);

} // end method _gather_info()



