// 
//  Copyright (C) 2008,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonApp.hh"
#include "TaskmonHelpMenu.hh"
#include "TaskmonRefDialog.hh"
#include "TaskmonUtils.hh"
#include "cxcds_version.h"

using namespace std;

TaskmonHelpMenu::TaskmonHelpMenu()
{
  //HELP (base_w)
    // Reference Manual (menuref)
    // Quick Hints (menuhints)
       // subhints
          // Interfaces (subhintsIface)
          //Preferences
    // FAQ (menuFaq)
    // About (menuAbout)
    //Acknowledgements (menuAck)
  //------------------

   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_Help");

   menu = gtk_menu_new();

   menuRef = gtk_menu_item_new_with_mnemonic("Reference _Manual");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuRef, pos++);

   //   menuHints = gtk_menu_item_new_with_mnemonic("Quick _Hints");
   //   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuHints, pos++);

   //     subHints = gtk_menu_new();
   //     gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuHints), subHints);

   //     subHintsIface = gtk_menu_item_new_with_label("Interfaces");
   //     gtk_menu_shell_insert(GTK_MENU_SHELL(subHints), subHintsIface, 0);

   //     subHintsPrefs = gtk_menu_item_new_with_label("Preferences");
   //     gtk_menu_shell_insert(GTK_MENU_SHELL(subHints), subHintsPrefs, 1);

   //   g_signal_connect_swapped (G_OBJECT(subHintsIface), "activate",
   //                             G_CALLBACK(hints_cb), (gpointer) 0);
//   g_signal_connect_swapped (G_OBJECT(subHintsPrefs), "activate",
//                             G_CALLBACK(hints_cb), (gpointer) 1);

   menuFaq = gtk_menu_item_new_with_mnemonic("_FAQ");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuFaq, pos++);

   menuAbout = gtk_menu_item_new_with_label("About");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuAbout, pos++);

#ifdef WLM_USE_ACK_MENU
   menuAck = gtk_menu_item_new_with_label("Acknowledgements");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuAck, pos++);
   g_signal_connect_swapped (G_OBJECT(menuAck), "activate",
                             G_CALLBACK(acknowledgement_cb),
                             (gpointer) NULL);
#else 
   menuAck = NULL; 
#endif

   g_signal_connect_swapped (G_OBJECT(menuRef), "activate",
                             G_CALLBACK(reference_manual_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT(menuFaq), "activate",
                             G_CALLBACK(faq_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT(menuAbout), "activate",
                             G_CALLBACK(about_cb),
                             (gpointer) NULL);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);

   GtkTooltips* tips = theApp->get_tips_group();
   if (tips)
   {
      gtk_tooltips_set_tip(tips, menuRef,
         "Display contents of the reference manual",NULL);
      //      gtk_tooltips_set_tip(tips, subHintsIface,
      //         "Display interface specific help",NULL);
      //      gtk_tooltips_set_tip(tips, subHintsPrefs,
      //         "Display help on configuring peg preferences",NULL);
      //      gtk_tooltips_set_tip(tips, menuFaq,
      //         "Display Frequently Asked Questions",NULL);
      gtk_tooltips_set_tip(tips, menuAbout,
         "Display version info",NULL);
#ifdef WLM_USE_ACK_MENU
      gtk_tooltips_set_tip(tips, menuAck,
         "Display acknowledgements",NULL);
#endif
   }
}

TaskmonHelpMenu::~TaskmonHelpMenu()
{
   
}

int TaskmonHelpMenu::show()
{
   gtk_widget_show_all(base_w); 
   return 0; 
}

int TaskmonHelpMenu::reference_manual_cb()
{
   string reffile;
   char *pth = getenv("ASCDS_INSTALL");
   if (pth)
   {
      reffile = pth;
      reffile.append("/doc/txt/");
   }
   reffile.append("taskmon_refman.txt");
   TaskmonRefDialog *dlg = new TaskmonRefDialog("Reference Manual", reffile);
   if (dlg)   dlg->show();
   return 0; 
}

int TaskmonHelpMenu::faq_cb()
{
   string reffile;
   char *pth = getenv("ASCDS_INSTALL");
   if (pth)
   {
      reffile = pth;
      reffile.append("/doc/txt/");
   }
   reffile.append("taskmon_faq.txt");
   TaskmonRefDialog *dlg = new TaskmonRefDialog("FAQ - Frequently Asked Questions", reffile);
   if (dlg)   dlg->show();

   return 0; 
}

int TaskmonHelpMenu::hints_cb(gpointer user_data)
{
   gint pos = GPOINTER_TO_INT(user_data); 
   if (pos < 0 || pos > 2)  return 1; 

   string title, reffile;
   char *pth = getenv("ASCDS_INSTALL");
   if (pth)
   {
      reffile = pth;
      reffile.append("/doc/txt/");
   }

   switch(pos)
   {
      case 0:    // interfaces 
        title = "Quick Hints - Interfaces";
        reffile.append("peg_qh_interfaces.txt");
      break; 
      case 1:
        title = "Quick Hints - Customizing Taskmon Preferences";
        reffile.append("peg_qh_preferences.txt");
      break; 
      default:
         return 1;
      break; 
   }
   TaskmonRefDialog *dlg = new TaskmonRefDialog(title, reffile);
   if (dlg)   dlg->show();
   return 0; 
}

int TaskmonHelpMenu::about_cb()
{
   gtk_show_about_dialog(NULL, 
      "name", "taskmonitor", 
      "title", "About Taskmonotor",
      "copyright", CXCDS_COPYRIGHT_STRING,
      "version", CXCDS_VERSION_STRING, 
      NULL);  
   return 0; 
}

int TaskmonHelpMenu::acknowledgement_cb()
{
   return 0; 
}


