// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonMenu.hh"
#include "TaskmonFileMenu.hh"
#include "TaskmonHelpMenu.hh"
#include "CiaoAnalysisMenu.hh"

using namespace std;

TaskmonMenu::TaskmonMenu(string filename, GtkTooltips* tips)
{
   base_w = gtk_menu_bar_new();
   menuFile = new TaskmonFileMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuFile->base_widget());

   menuAnalysis = new CiaoAnalysisMenu(filename, tips);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuAnalysis->base_widget());

   menuHelp = new TaskmonHelpMenu();
   gtk_menu_item_set_right_justified(GTK_MENU_ITEM(menuHelp->base_widget()),
      TRUE);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuHelp->base_widget());
}

TaskmonMenu::~TaskmonMenu()
{
   if (menuFile)
   {
      delete menuFile;
      menuFile = NULL;
   }
   if (menuAnalysis)
   {
      delete menuAnalysis;
      menuAnalysis = NULL;
   }
   if (menuHelp)
   {
      delete menuHelp;
      menuHelp = NULL;
   }
}

int TaskmonMenu::show()
{
   menuFile->show();
   menuHelp->show();
   menuAnalysis->show();
   gtk_widget_show(base_w);
   return 0; 
}


