// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonPrintFileDialog.hh"
#include "TaskmonApp.hh"
#include <iostream>
#include <unistd.h>

//const gchar* text_buffer_contents;
 string filespec;
 int pid;
 string buf_contents;
 string _print_command;
 ofstream outfile;
 gint result;
 int status=0;

using namespace std;

TaskmonPrintFileDialog::TaskmonPrintFileDialog()
{
  // This dialog box has three sections: 
  //         An output selection ( a label and 2 check buttons)
  //         A Printer Command section (label and a GTKEntry widget)
  //         A button bar at the bottom.

  // create the starting dialog box container
   _dialog = gtk_dialog_new_with_buttons("Print File", 
					 NULL,
					 GTK_DIALOG_DESTROY_WITH_PARENT, 
					 GTK_STOCK_OK, GTK_RESPONSE_OK, 
					 GTK_STOCK_APPLY, GTK_RESPONSE_APPLY, 
					 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					 NULL);

 // Section 1 - Output section. Make it a 4x8 table and pack the 
 // label and two check buttons in a row

   // create the 3x8 table
   _output_table = gtk_table_new(4,8, true);

   //create the label and pack it in row 2, left of the table
   _output_label = gtk_label_new("Output:"); 
   gtk_table_attach_defaults(GTK_TABLE(_output_table), _output_label, 0,2,1,2);

   // Now create two check buttons and pack them in row 3, #2 left
   _output_commandCHKB = gtk_check_button_new_with_label ("Command");
   gtk_toggle_button_set_active((GtkToggleButton*)_output_commandCHKB, TRUE);
   gtk_table_attach_defaults(GTK_TABLE(_output_table),
			     _output_commandCHKB,
			     2,4,
			     2,3);

   _output_resultsCHKB = gtk_check_button_new_with_label ("Results");
   gtk_toggle_button_set_active((GtkToggleButton*)_output_resultsCHKB, TRUE);
   gtk_table_attach_defaults(GTK_TABLE(_output_table),
			     _output_resultsCHKB,
			     5,7,
			     2,3);

   // Now pack the whole Section one into the dialog vbox
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), _output_table);


 // Pack a separator in between section 1 and section 2

   _sep1 = gtk_hseparator_new();
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), _sep1);
 

 // Section 2 - Printer command section. Make it a 3x8 table and pack the 
 // label and the entry widget

   GtkWidget* lab_w = gtk_label_new("Printer Command:  "); 

   GtkWidget* tab = gtk_table_new(3,8, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), tab);

   // create the text entry widgetm inser tit inthe table and 
   //initialize the field to "lpr "
   _entry_w = gtk_entry_new(); 
   gtk_table_attach_defaults(GTK_TABLE(tab), lab_w, 1,3,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), GTK_WIDGET(_entry_w), 3,7,1,2);
   gtk_entry_set_text( (GtkEntry*)_entry_w, "lpr ");

   // render it all
   gtk_widget_show_all(_dialog);

   //
   // PROCESSING
   //
   // At least ONE of the check boxes up top have to be set
   // check them. and if one is not set, pop up a message dialog 
   // and stay in the while loop until one is set

    // first wait for a button click and then
    // gather the information on the buttons and File selection
     result = gtk_dialog_run (GTK_DIALOG (_dialog));
     status = _gather_info();

     // if the button pressed was CANCEL, then kill the widget and bail

     if(result == GTK_RESPONSE_CANCEL)
        gtk_widget_destroy (_dialog);
     else
       {

     // Check the returned values of the two buttons - if both zero 
     // stay here until one is set
       while( (_o_cCHKBval == 0) && 
	      (_o_rCHKBval == 0) &&
	      (result != GTK_RESPONSE_CANCEL) )
	 {
	   TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
	   _error_dialog->_pop_error_up("You must have at least one, or both, of the checkboxes checked indicating what text field you wish to have written to the file.");
         // Drop into the blocking loop and await a user click on
         // one of the buttons at the bottom
           result = gtk_dialog_run (GTK_DIALOG (_dialog));
           status = _gather_info();
	   delete _error_dialog;
	 } // end while( (<_o_cCHKBval == 0) && (_o_rCHKBval == 0) )

       _print_command = text_buffer_contents;

     //now be sure the user entered a print command
       while( (_print_command.size() == 0) &&
	      (result != GTK_RESPONSE_CANCEL) )
	 {
	   TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
	   _error_dialog->_pop_error_up("You must specify a print command, such as lpr, in the text field.");
         // Drop into the blocking loop and await a user click on
         // one of the buttons at the bottom
           result = gtk_dialog_run (GTK_DIALOG (_dialog));
           status = _gather_info();
           _print_command = text_buffer_contents;
	   delete _error_dialog;
	 } // end while( (filespec == NULL )

   //  One of the buttons was pressed. If it was APPLY, gather all the info
   // and execute the print via system, then get back into the RUN loop
   while(result == GTK_RESPONSE_APPLY)
     {
      // Gather the check button and print command values
      status = _gather_info();

      // Next write out a temporary file containing the info indicated by 
      // the values of the check buttons

      filespec = _tempfile_envar_path();

      // You now have a clean path. Concatenate a file name to it

      pid = (int) getpid();
      filespec.append("/tmonitor_");
      filespec.append(_int_to_string(pid) );
      filespec.append("pid.txt");

      // now write one or both of the buffers contents to the temp file
      // then print the temp file out

       outfile.open(filespec.c_str(), ios_base::out);

       if (outfile)    // output file sucessfully opened
        {
         if ( _o_cCHKBval == 1)
         {
            outfile << "\n COMMAND : \n --------- \n" << endl;
	    buf_contents = theApp->_commandArea->get_buffer_contents();
            outfile << buf_contents << endl;
         }

         if ( _o_rCHKBval == 1)
         {
            outfile << "\n RESULTS : \n --------- \n" << endl;
	    buf_contents = theApp->_outputArea->get_buffer_contents();
            outfile << buf_contents << endl;
         }
        outfile.close();

        // now launch the print command
	_print_command = text_buffer_contents; 
	_print_command.append(" ");
	_print_command.append(filespec);

        // submit the print job to the user-specified printer
	system(_print_command.c_str());

        // destroy the temporary file
	_print_command = "rm ";
	_print_command.append(filespec);
	system(_print_command.c_str());
	
	}
     else  //else the file open failed...bail
       {
       TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
       _error_dialog->_pop_error_up("Unable to create the temporary file necessary to print.");
       delete _error_dialog;
       }
     
      // Drop back into the blocking loop and await a button click 
      result = gtk_dialog_run (GTK_DIALOG (_dialog));

     } // end while(result == GTK_RESPONSE_APPLY)

   // Either the user clicked APPLY first, and we just came out 
   // of the while loop, because the user clicked OK or CANCEL. 
   // Or the user clicked OK or CANCEL as thier first click. Either
   // way process the clicks

  switch (result)
    {
      case GTK_RESPONSE_OK:
	{// Gather the check button and print command values
           status = _gather_info();

      // Next write out a temporary file containing the info indicated by 
      // the values of the check buttons

        filespec = _tempfile_envar_path();

      // You now have a clean path. Concatenate a file name to it
        pid = (int) getpid();
        filespec.append("/tmonitor_");
        filespec.append(_int_to_string(pid) );
        filespec.append("pid.txt");
      // now write one or both of the buffers contents to the temp file
      // then print the temp file out

       outfile.open(filespec.c_str(), ios_base::out);
         if (outfile)    // output file sucessfully opened
          {
           if ( _o_cCHKBval == 1)
            {
              outfile << "\n COMMAND : \n --------- \n" << endl;
	      buf_contents = theApp->_commandArea->get_buffer_contents();
              outfile << buf_contents << endl;
            }

         if ( _o_rCHKBval == 1)
           {
            outfile << "\n RESULTS : \n --------- \n" << endl;
	    buf_contents = theApp->_outputArea->get_buffer_contents();
            outfile << buf_contents << endl;
           }
         outfile.close();

        // now launch the print command
	  _print_command = text_buffer_contents; 
	  _print_command.append(" ");
	  _print_command.append(filespec);


        // submit the print job to the user-specified printer
	  system(_print_command.c_str());

        // destroy the temporary file
	  _print_command = "rm ";
	  _print_command.append(filespec);
	  system(_print_command.c_str());
       }
     else  //else the file open failed...bail
       {
       TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
       _error_dialog->_pop_error_up("Unable to create the temporary file necessary to print.");
       delete _error_dialog;
       }
 
	}  // end Gather the check button and print command values

        gtk_widget_destroy (_dialog);
         break;
    case GTK_RESPONSE_CANCEL:
        gtk_widget_destroy (_dialog);
         break;

      default:
	cout<<endl<<"DO NOTHING"<<endl;
         break;
    }
  }// end RESULT  was not CANCEL


} // end TaskmonPrintFileDialog()

TaskmonPrintFileDialog::~TaskmonPrintFileDialog()
{
   gtk_widget_destroy(_dialog);
}

void TaskmonPrintFileDialog::create_dialog()
{
   _dialog = gtk_dialog_new_with_buttons("Print File", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(3,8, true);
   GtkWidget* lab_w = gtk_label_new("Printer Command:"); 
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), tab);
   _entry_w = gtk_entry_new(); 
   gtk_table_attach_defaults(GTK_TABLE(tab), lab_w, 1,3,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), GTK_WIDGET(_entry_w), 3,7,1,2);
   //   show(); 
}



int TaskmonPrintFileDialog::doit()
{
   create_dialog();
   if (gtk_dialog_run(GTK_DIALOG(_dialog)) == GTK_RESPONSE_OK)
   {
      // perform print 
      print_file(); 
   }
   gtk_widget_destroy(_dialog);
   return 0; 
}


int TaskmonPrintFileDialog::print_file()
{
   string txt = gtk_entry_get_text(GTK_ENTRY(_entry_w));
   string fname = theApp->get_file_path(); 

   if (fname.length() && (access(fname.c_str(),F_OK) == 0))
   {
      if (txt.length() == 0)
         _cmd = "lpr " + fname; 
      else
         _cmd = "cat " + fname + " | " + txt;
      //zzzzzzzz
      theApp->output_message("Sending printer request to print file " + fname); 
      system(_cmd.c_str()); 
   }
   else 
   {
      string msg = "Error: Print request failed. Unable to access file " + fname; 
      //zzzzzzzzzzzz
      theApp->output_message(msg); 
      return 1; 
   }
   return 0; 
}


int TaskmonPrintFileDialog::_gather_info()
{
      // Gather the check button values
      _o_cCHKBval = gtk_toggle_button_get_active((GtkToggleButton*)_output_commandCHKB);
      _o_rCHKBval = gtk_toggle_button_get_active((GtkToggleButton*)_output_resultsCHKB);

      text_buffer_contents = gtk_entry_get_text((GtkEntry*)_entry_w);


  return(0);

} // end method _gather_info()


