// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonTextArea.hh"

using namespace std;

TaskmonTextArea::TaskmonTextArea()
{
   base_w = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (base_w),
                                   GTK_POLICY_ALWAYS,
                                   GTK_POLICY_ALWAYS);
   buffer = gtk_text_buffer_new(NULL);
   textBox = gtk_text_view_new_with_buffer(buffer);
   gtk_text_view_set_editable(GTK_TEXT_VIEW(textBox), false);
   gtk_container_add(GTK_CONTAINER(base_w), textBox);
   //   prepend = true; 
   prepend = false; 
}


TaskmonTextArea::~TaskmonTextArea()
{
   
}


int TaskmonTextArea::add_message(string msg)
{
   if (msg.length())
   {
    GtkTextIter iter;
      gtk_text_buffer_get_end_iter(buffer, &iter);
      gtk_text_buffer_insert(GTK_TEXT_BUFFER(buffer), &iter, msg.c_str(), -1);
   }
   // Set the start iter to the beginning of the buffer for later use in
   // the FIND command
      gtk_text_buffer_get_start_iter(buffer, &start_iter);

   return 0; 
}

string TaskmonTextArea::get_buffer_contents()
{
  string buffer_contents = "Test Buffer contents";
  GtkTextIter startiter, enditer;

  gtk_text_buffer_get_start_iter (buffer, &startiter);
  gtk_text_buffer_get_end_iter (buffer, &enditer);

  buffer_contents = gtk_text_buffer_get_text(buffer, 
					     &startiter,
					     &enditer,
					     FALSE);

  return(buffer_contents);


} // end method :get_buffer_contents()


int TaskmonTextArea::show()
{
   gtk_widget_show(base_w);
   return 0;
}

void TaskmonTextArea::get_text_start_iter(GtkTextBuffer *buffer, 
					  GtkTextIter *iter)
{
 gtk_text_buffer_get_start_iter (buffer, iter);
}

void TaskmonTextArea::get_text_end_iter(GtkTextBuffer *buffer, 
					GtkTextIter *iter)
{
 gtk_text_buffer_get_end_iter (buffer, iter);
}


void TaskmonTextArea::MoveToString(string searchstring)
{
  gboolean foundit;

  gtk_text_buffer_get_end_iter (buffer, &end_iter);

  foundit = gtk_text_iter_forward_search(&start_iter,
			       searchstring.c_str(),
			       GTK_TEXT_SEARCH_TEXT_ONLY,
			       &matchstart,
			       &matchend,
			       &end_iter);

  if(foundit)
    {gtk_text_view_scroll_to_iter ((GtkTextView*) textBox,
				 &matchstart,
				 0.0,        // mid margin
				 FALSE,
				 0.0,
				 0.0);

     gtk_text_buffer_select_range(buffer, 
      				       &matchstart, 
      				       &matchend);


     // since you found the string, move the start iter to the 
     // end of the string so that subsequent FIND commands can Find Next
        start_iter = matchend;

    } // end if (foundit)
  else // else the string was not found between the iters
    {
     // Tell the user that the string cannot be found between the iters
       TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
       _error_dialog->_pop_error_up("Unable to locate the specified string.");
       delete _error_dialog;

     // move the iter to the start of the buffer. It might have been
     // somewhere inside the buffer
       gtk_text_buffer_get_start_iter (buffer, &start_iter);
    }


} // end TaskmonTextArea::MoveToString


void TaskmonTextArea::MoveToEnd()
{
  GtkTextIter end_iter;
  GtkTextMark* endmark;

 

  gtk_text_buffer_get_end_iter (buffer, &end_iter);

  endmark = gtk_text_buffer_create_mark(buffer,
  					"mymark",
  					&end_iter,
  					TRUE);
  
  gtk_text_view_scroll_mark_onscreen((GtkTextView*) textBox, endmark);


} // end method TaskmonTextArea::MoveToEnd()
