// 
//  Copyright (C) 1998,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CIAOGUIIPCCOMM_HH
#define	CIAOGUIIPCCOMM_HH

#include <fcntl.h>
#include <unistd.h>
#include <string>
#include <string.h>
#include <stdlib.h>

extern "C"
{
#if defined(__sun) || defined(__alpha)
extern          int usleep(unsigned int useconds);
#endif
}

#if defined(_CIAOGUIIPC_LIBRARY_)
#include	"xpa.h"
#endif


#include "CiaoGuiIPCMessage.hh"


// Wait some number of seconds before attempting to re-launch the application
#define	COMM_WAIT_FOR_LAUNCH_SECONDS	        30
#define CiaoGuiIPC_MAX_MESSAGE_LENGTH          150

// Structure used to keep track of callbacks, internally.

typedef struct _callbacks
{
   char    key[CiaoGuiIPC_KEY_LENGTH], name[128];
   CiaoGuiIPCMessage*		(*cb)(CiaoGuiIPCMessage&);
   struct  _callbacks	*next;
} CiaoGuiIPCCallbackTable, *CiaoGuiIPCCallbackTablePtr;

#if !defined(_CIAOGUIIPC_LIBRARY_)
struct XPA;
#endif

// The Interprocess Communication Base Class. Contains all routines
// and data common to both Clients and Servers

class CiaoGuiIPCComm
{
   public:
      CiaoGuiIPCComm(char* pname, char **args=NULL);
      virtual ~CiaoGuiIPCComm();

      int     SendWithReply(char *dest, CiaoGuiIPCMessage&, CiaoGuiIPCMessage&,
                            int launch=1, char *alias=NULL, char** argv=NULL);
      int     Send(char *dest, CiaoGuiIPCMessage&, int launch=1, 
                   char *alias=NULL, char** argv=NULL);
      void    EventLoop();
      void    Poll(int msec);
      int     Launch(char*, char** argv = NULL, char *alias=NULL);
      int     AlreadyRunning(char*);
      
      int     WaitForLaunch(char *);
      int     LaunchedApp() { return _launched_app;}
      int                     XpaIsSetup() { return (pxpa != NULL);}

   protected:
      string  name;
      char    **envp, **argv;

#if defined(_CIAOGUIIPC_LIBRARY_)
      XPA     pxpa;
#else
      XPA     *pxpa;
#endif

   private:
      void    Initialize(char*, char**);
      int     CheckParameters(char *, CiaoGuiIPCMessage&);
      int     waittime;
      int     _launched_app; //non-zero= launched application
};

// Messages for all applications

#define EXIT                            "exit"
#define QUIT                            "quit"

#endif
