// 
//  Copyright (C) 1998,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef CIAOGUIIPCMESSAGE_HH
#define CIAOGUIIPCMESSAGE_HH


using namespace std; 

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <string>
#include <fstream>
#include <iostream>
#include <strstream>


// macro for duplicating a string
#define STRDUP(_dst, _src) \
{                          \
   if(_src)                \
   {                       \
      _dst = (char*) malloc(strlen(_src) + 1);\
      strcpy(_dst, _src);  \
   }                       \
}


// General Constants
#define CiaoGuiIPC_CALLBACK_NAME_MAX   128
#define CiaoGuiIPC_KEY_LENGTH           64
#define CiaoGuiIPC_PARAM_LIMIT        1024

// When converting floats and doubles to strings, this is the maximum
// number of bytes that a float/double will convert to.
#define CiaoGuiIPC_MESSAGE_MAX_BYTES   128



class CiaoGuiIPCMessage
{
   public:
      CiaoGuiIPCMessage();
      CiaoGuiIPCMessage(char*);
      CiaoGuiIPCMessage(string);
      ~CiaoGuiIPCMessage();

      int Clear();

      int SetParam(char*);

      int AddParam(string&);
      int AddParam(float);
      int AddParam(double);
      int AddParam(int);
      int AddParam(long);
      int AddParam(char*);

      string GetKey();
      char *GetMessage(int index=0);
      int GetTotalParams(){return nEntries;}
      int GetLength(int index=0);

      int GetParam(int, string&);
      int GetParam(int, float&);
      int GetParam(int, double&);
      int GetParam(int, int&);
      int GetParam(int, long&);

      CiaoGuiIPCMessage& operator=(const CiaoGuiIPCMessage &msg);

   protected:
   private:
      void Init();

      char key[CiaoGuiIPC_PARAM_LIMIT];
      char **args;
      int nEntries, nSlots;
};

#endif
