// 
//  Copyright (C) 2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


//     This file contains the definition for the CiaoToolEntry class. The
//     class is used to store command / menu components for the Analysis
//     Pulldown menu.


#ifndef CIAOTOOLENTRY_HH
#define CIAOTOOLENTRY_HH

#include <string>
#include <vector>

#include "CiaoGuiDefsP.hh"

using namespace std; 

class CiaoToolEntry
{ 

public:
   CiaoToolEntry(int depth);                           // separator 
   CiaoToolEntry(string name, string tip, int depth);    // category  
   CiaoToolEntry(string name, string txt, int format,    // help  
             int depth);    
   CiaoToolEntry(string name, string tip, string ftype,   // tool/command 
             string ctype, string cmd, int depth);      

   ~CiaoToolEntry(); 
   string getName()     { return _name; }
   string getTip()      { return _tip;  }
   string getCmdEval()  { return _cmdEval;  }
   string getHelp()     { return _help; }
   int getType()        { return _type; } 
   int getDepth()       { return _depth; } 
   int getSupported()   { return _supported; }
   int isSupported()    { return ((_supported == SUPPORTED) && 
                                  (_supTemp == SUPPORTED)); }

   void set_replace_filename_method(funcPtr fp)  { _replace_filename = fp; } 
   void set_status_message_method(statusFuncPtr sfp)  { _status_message = sfp; } 

   int  DisableUnsupported(); 
   void ReconfigureCmd();
   void ExpandMacros(string val);
   void SupportByFormat(string type);

   // constants - menu entry types 
   static const int CATEGORY_TYPE; 
   static const int COMMAND_TYPE; 
   static const int SEPARATOR_TYPE; 
   static const int HELP_TYPE; 
   // constants - menu entry support 
   static const int SUPPORTED;       // entry should be active in menu
   static const int NOT_SUPPORTED;   // entry should be grayed out of menu
   static const int NEVER_SUPPORTED; // entry should not be in menu

   void doit(); 

protected: 

   string _name;        // AMDL menu label 
   string _cmd;         // AMDL commandline 
   string _template;    // AMDL file templates 
   string _cmdType;     // AMDL command type - menu | bind 
   string _tip;         // tool tip  
   string _cmdEval;     // evaluated commandline to execute  
   string _help;        // help information 
   vector<string> _formats; // vector of supported file formats  
   int    _type;        // the type of item- category|command|separator|help
   int    _supported;   // Support of the item by ciao apps
   int    _supTemp;     // support based on file template 
   int    _depth;
   funcPtr _replace_filename;  // method to replace filename macro 
   statusFuncPtr _status_message; // method to set status message  
};  

#endif 
