// 
//  Copyright (C) 2003,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


//     This file contains the definition for the CiaoToolList class. The
//     class is used to configure the Analysis pulldown.

#ifndef CIAOTOOLLIST_HH
#define CIAOTOOLLIST_HH

#define CTL_MAX_DEPTH  10

class CiaoToolEntry; 

#include "CiaoToolEntry.hh"
#include <list>
#include <string>

class CiaoToolList {

  public:

   CiaoToolList(string name);
   ~CiaoToolList();

   int  isValidList()             { return (_parsed_OK == 0); }   
   list<CiaoToolEntry*>& getList() { return _list; }
   string getFilename()           { return _fname; }
   int  parseList();  
   void SupportByFormat(char* type);

   // constants - error status conditions
   static const int CTL_NO_ERR;         // no error 
   static const int CTL_NO_FILE_ERR;    // no file specified  
   static const int CTL_OPEN_ERR;       // unable to open file  
   static const int CTL_BAD_CAT_ERR;    // no category specified  
   static const int CTL_CAT_ALLOC_ERR;  // unable to allocate category memory 
   static const int CTL_CMD_ALLOC_ERR;  // unable to allocate command memory 
   static const int CTL_HMENU_ERR;      // incorrect hmenu/endhmenu sequence 
   static const int CTL_HELP_ALLOC_ERR; // unable to allocate help memory 
   static const int CTL_SEP_ALLOC_ERR;  // unable to allocate separator memory 
   static const int CTL_EMPTY_BUF_ERR;  // no buffer to parse   
   static const int CTL_BAD_TYPE_ERR;   // command type is not menu/bind
   static const int CTL_PARSE_SEQ_ERR;  // parse ended in unexpected state

protected:

   int  addSeparator(); 
   int  parseCategory(char* buf);  
   int  parseEntry();
   int  parseHelp();
   int  parseLine(char* buf);  
   void removeComment(char *buf); 

   list<CiaoToolEntry*> _list;  // List of entries from tool table 

   // parser states 
   typedef enum CTL_PARSE {INITIAL, PARAM, TOOLNAME, TEMPLATES, CMDTYPE,
                           COMMAND, TIP, HELP};

   string   _fname; 
   CTL_PARSE _state; 
   int       _parsed_OK; 
   int       _depth; 
   int       _hmenuCnt; 

   // buffers
   string   _label;
   string   _templates;
   string   _type;
   string   _command; 
   string   _misc; 
   string   _help; 
};

#endif 
