// 
//  Copyright (C) 2002,2003,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "Parameter.hh"

#include "PegApp.hh"
#include "PegParEntryBoolean.hh"
#include "PegParData.hh"
//WLM #include "PegPopupMenu.hh"


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
PegParEntryBoolean::PegParEntryBoolean(ParamBoolean *item):
   PegParEntry(item)
{
   _toggle = 1; 
   _entry_type = PegParEntry::BOOLEAN; 
   _item = new ParamBoolean(*item); 
   _yes_btn = gtk_radio_button_new_with_label(NULL, "Yes");
   _no_btn = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_yes_btn), "No");

   g_signal_connect(_yes_btn,"toggled", G_CALLBACK(ToggleChangeCB),(void*)this);
   g_signal_connect(_ind_txt,"activate", G_CALLBACK(IndirEditCB), (void*)this);
   g_signal_connect(_ind_txt, "focus-out-event", G_CALLBACK(FocusOutCB),
                    (void*)this);

   if (_tip_grp)
   {
      string prompt = _item->GetPrompt(); 
      gtk_tooltips_set_tip(_tip_grp, _yes_btn, prompt.c_str(), NULL);
      gtk_tooltips_set_tip(_tip_grp, _no_btn, prompt.c_str(), NULL);
      gtk_tooltips_set_tip(_tip_grp, _ind_txt, prompt.c_str(), NULL);
   }

   char *rcur = _item->GetRedirectCurrent();
   string  tmp = "\0"; 
   if (rcur[0] != '\0')
   {
      _disp_type = PegParEntry::INDIR;
      tmp.append(rcur);
   }
   else
   {
      _disp_type = PegParEntry::DEFAULT;
      if ((int)_item->GetCurrent())
         tmp.append("1");
      else  
         tmp.append("0");
   }
   set(tmp.c_str());

   initEntry();

   gtk_box_pack_start(GTK_BOX(_entry), _yes_btn, false, false, 0);
   gtk_box_pack_start(GTK_BOX(_entry), _no_btn, false, false, 0);
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
PegParEntryBoolean::~PegParEntryBoolean()
{
   if (_item)
   {
      delete _item;
      _item = NULL; 
   }
}


// --------------------------------------------------------
// Set value of toggle or indirection to specified value
// --------------------------------------------------------
void PegParEntryBoolean::set(const char* val)
{
   string  setting;

   if (val)
   {
      if (val[0] == ')' && (_disp_type != PegParEntry::INDIR))
         toggleDispType();

      if (_disp_type == PegParEntry::DEFAULT)
      {
         // NOTE: for now, since ParamBoolean does not deal w/ INDEF
         //       only deal w/ T, F. 
         if ((val[0] == '1') || (!_cxcparam_strcmp_loose(val, "yes")))
         {
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_yes_btn), true);
            setting = "yes";
         }
         else if ((val[0] == '0') || (!_cxcparam_strcmp_loose(val, "no"))) 
         {
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_no_btn), true);
            setting = "no";
         }
/**
         else 
         {
            _yes_btn->SetState(False, False);
            _no_btn->SetState(False, False);
            setting = "\0";
         }
**/
      }
      else if (_disp_type == PegParEntry::INDIR)
      {
         string  cur = gtk_entry_get_text(GTK_ENTRY(_ind_txt));
         if (cur != val)  gtk_entry_set_text(GTK_ENTRY(_ind_txt), (char*)val);
         if (thePData && _item)
         {
            Parameter *par = NULL;
   
            if ((par = thePData->GetParameter()) != NULL)
            {
               string  eval = par->EvalIndir((char*)_name.c_str(), (char*)val);
               gtk_entry_set_text(GTK_ENTRY(_ind_val), eval.c_str());
               setting = val;
            }
         }
      }
      if (thePData)
      {
         Parameter *par = thePData->GetParameter();
         int error = par->SetValueAtPosition(_item->GetPosition(), setting.c_str());
	 _prevState = setting;
         if (_disp_type == PegParEntry::INDIR)
            if (error != 1)
	       set_error_color();
	    else 
	       clear_error_color(); 
      }
   }
}


// --------------------------------------------------------
// Get value of toggle or indirection of current entry
// --------------------------------------------------------
char* PegParEntryBoolean::get()
{
   static string  val;

   switch(_disp_type)
   {
      case PegParEntry::DEFAULT:
         if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_yes_btn))) 
            val = "yes";
         else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_no_btn))) 
            val = "no";
         else 
            val = "\0"; 
      break;
      case PegParEntry::INDIR:
         val = gtk_entry_get_text(GTK_ENTRY(_ind_txt));
      break;
      default:
         val = _item->GetValue();
      break;
   }
   return (char*)val.c_str();
}


// --------------------------------------------------------
// Display the approriate widget for the data value
// --------------------------------------------------------
void PegParEntryBoolean::manage()
{
   gtk_widget_show(_lbl_w);

   switch(_disp_type)
   {
      case PegParEntry::INDIR:
	 gtk_widget_show_all(_ind_box);
         gtk_widget_hide_all(_entry_box);
      break;
      case PegParEntry::DEFAULT:  // fallthrough intended
      default:
         gtk_widget_show_all(_entry_box);
         gtk_widget_hide_all(_ind_box);
      break;
   }

   // ensure internal param value is updated when disp type changes
   string  tmp = get();
   if ((_disp_type != PegParEntry::INDIR) || (tmp.length())) 
      set(tmp.c_str()); 
}



// --------------------------------------------------------
// Callback to handle setting of Indirections
// --------------------------------------------------------
int PegParEntryBoolean::IndirEditCB(GObject *object, gpointer user_data)
{
   PegParEntryBoolean *obj = (PegParEntryBoolean*)user_data;
   if (obj) return obj->update_indir();
      else return 0;
}


int PegParEntryBoolean::update_indir()
{
   Parameter *par = NULL;
   string val = gtk_entry_get_text(GTK_ENTRY(_ind_txt));

   if (val == "\0")
   {
      string val = gtk_entry_get_text(GTK_ENTRY(_ind_txt));
   }
   else if (val.find(")") != 0)
   {
      string  msg = "Indirection for parameter " + _name +
         " must begin with )";
      set_error_color(); 
      theApp->status_message(msg);
   }
   else if ((par = thePData->GetParameter()) != NULL)
   {
      if (par->SetValueAtPosition(_item->GetPosition(), val) != 1)
         set_error_color(); 
      else
      {
         clear_error_color();
         string evl = par->EvalIndir((char*)_name.c_str(), (char*)val.c_str());
         if (evl.length())
            gtk_entry_set_text(GTK_ENTRY(_ind_val), evl.c_str());
         else
            gtk_entry_set_text(GTK_ENTRY(_ind_val), "\0");
      }
   }
   set(val.c_str());
   return 0; 
}


// callback to handle leaving focus event from text|indirection field
int PegParEntryBoolean::FocusOutCB(GObject *object, GdkEvent *event,
   gpointer user_data)
{
   PegParEntryBoolean *obj = (PegParEntryBoolean*)user_data;
   GtkWidget* ww = GTK_WIDGET(object);
   if (obj)
      if (ww == obj->GetIndirField())         return obj->update_indir();
//         else if (ww == obj->GetTextField())  return obj->update_text();
   return 0;
}


// --------------------------------------------------------
// Callback to handle Toggle values changing
// --------------------------------------------------------
int PegParEntryBoolean::ToggleChangeCB(GObject *object, gpointer user_data)
{
   PegParEntryBoolean *obj = (PegParEntryBoolean*)user_data;

   if (obj)
   {
      // this is called when the old value is unset and the new one is set
      // so check toggleState() to only handle the new setting. 
      if (obj->getDispType() == PegParEntry::DEFAULT)  
      {
         string  state = obj->GetPrevState(); 
         if (obj->ToggleState())
         {
            if (strcmp("no", obj->get()))
            {
               if (_cxcparam_strcmp_loose(state, "yes"))  obj->set("1");
            }
            else if (_cxcparam_strcmp_loose(state,"no"))  obj->set("0");
         }
         else if ((state == "\0") || (state.find(")") == 0))  
         { 
            // changed from indir w/ no prev toggle set - so handle new
            // setting now 
            obj->set(obj->get());
            obj->ToggleState();  // update toggle state
         }
      }
   }
   return 0; 
}


void PegParEntryBoolean::set_error_color()
{
   gtk_widget_modify_bg(_ind_txt, GTK_STATE_NORMAL, &_errcolor);
}


void PegParEntryBoolean::clear_error_color()
{
   gtk_widget_modify_bg(_ind_txt, GTK_STATE_NORMAL, NULL);
}


